"""Testcases for cssutils.css.CSSValue and CSSPrimitiveValue."""

# from decimal import Decimal # maybe for later tests?
import xml.dom
import basetest
import cssutils
import types


class PropertyValueTestCase(basetest.BaseTestCase):

    def setUp(self):
        self.r = cssutils.css.PropertyValue()

    def test_init(self):
        "PropertyValue.__init__() .item() .length"
        pv = cssutils.css.PropertyValue()
        self.assertEqual(u'', pv.cssText)
        self.assertEqual(0, pv.length)
        self.assertEqual(u'', pv.value)

        cssText = u'0, 0/0 1px var(x) url(x)'
        items = [u'0', u'0', u'0', u'1px', u'var(x)', 'url(x)']
        pv = cssutils.css.PropertyValue(cssText)
        self.assertEqual(cssText, pv.cssText)
        self.assertEqual(6, len(pv))
        self.assertEqual(6, pv.length)

        # __iter__
        for i, x in enumerate(pv):
            self.assertEqual(x.cssText, items[i])

        # cssText
        for i, item in enumerate(items):
            self.assertEqual(item, pv[i].cssText)
            self.assertEqual(item, pv.item(i).cssText)

    def test_cssText(self):
        "PropertyValue.cssText"
        tests = {
                 u'0': (None, 1, None),
                 u'0 0': (None, 2, None),
                 u'0, 0': (None, 2, None),
                 u'0,0': (u'0, 0', 2, None),
                 u'0  ,   0': (u'0, 0', 2, None),
                 u'0/0': (None, 2, None),
                 u'/**/ 0 /**/': (None, 1, u'0'),
                 u'0 /**/ 0 /**/ 0': (None, 3, u'0 0 0'),
                 u'0, /**/ 0, /**/ 0': (None, 3, u'0, 0, 0'),
                 u'0//**/ 0//**/ 0': (None, 3, u'0/0/0'),
                 u'/**/ red': (None, 1, u'red'),
                 u'/**/red': (u'/**/ red', 1, u'red'),
                 u'red /**/': (None, 1, u'red'),
                 u'red/**/': (u'red /**/', 1, u'red'),

                 u'a()1,-1,+1,1%,-1%,1px,-1px,"a",a,url(a),#aabb44': (
                 u'a() 1, -1, +1, 1%, -1%, 1px, -1px, "a", a, url(a), #ab4',
                    12, u'a() 1, -1, +1, 1%, -1%, 1px, -1px, "a", a, url(a), #ab4'),

                # calc values
                 u'calc(1)': (None, 1, u'calc(1)'),
                 u'calc( 1)': (u'calc(1)', 1, u'calc(1)'),
                 u'calc(1 )': (u'calc(1)', 1, u'calc(1)'),
                 u'calc(1px)': (None, 1, u'calc(1px)'),
                 u'calc(1p-x-)': (None, 1, u'calc(1p-x-)'),
                 u'calc(1%)': (None, 1, u'calc(1%)'),
                 u'calc(-1)': (None, 1, u'calc(-1)'),
                 u'calc(+1)': (None, 1, u'calc(+1)'),
                 u'calc(1  +   1px)': (u'calc(1 + 1px)', 1, u'calc(1 + 1px)'),
                 u'calc(1 - 1px)': (None, 1, u'calc(1 - 1px)'),
                 u'calc(1*1px)': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
                 u'calc(1  /  1px)': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
                 u'calc( 1*1px)': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
                 u'calc( 1  /  1px)': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
                 u'calc(1*1px )': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
                 u'calc(1  /  1px )': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
                 u'calc( 1*1px )': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
                 u'calc( 1  /  1px )': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
                 u'calc(var(X))': (None, 1, None),
                 u'calc(2 * var(X))': (None, 1, None),
                 u'calc(2px + var(X))': (None, 1, None),

                 #issue #24
                 u'rgb(0, 10, 255)': (None, 1, u'rgb(0, 10, 255)'),
                 u'hsl(10, 10%, 25%)': (None, 1, u'hsl(10, 10%, 25%)'),
                 u'rgba(0, 10, 255, 0.5)': (None, 1, u'rgba(0, 10, 255, 0.5)'),
                 u'hsla(10, 10%, 25%, 0.5)': (None, 1, u'hsla(10, 10%, 25%, 0.5)'),

                #issue #27
                u'matrix(0.000092, 0.2500010, -0.250000, 0.000092, 0, 0)': (
                    u'matrix(0.000092, 0.250001, -0.25, 0.000092, 0, 0)', 1,
                    u'matrix(0.000092, 0.250001, -0.25, 0.000092, 0, 0)')
                 }
        for (cssText, (c, l, v)) in tests.items():
            if c is None:
                c = cssText
            if v is None:
                v = c

            pv = cssutils.css.PropertyValue(cssText)
            self.assertEqual(c, pv.cssText)
            self.assertEqual(l, pv.length)
            self.assertEqual(v, pv.value)

        tests = {
                u'0 0px -0px +0px': (u'0 0 0 0', 4),
                u'1 2 3 4': (None, 4),
                u'-1 -2 -3 -4': (None, 4),
                u'-1 2': (None, 2),
                u'-1px red "x"': (None, 3),
                u'a, b c': (None, 3),
                u'1px1 2% 3': (u'1px1 2% 3', 3),
                u'f(+1pX, -2, 5%) 1': (u'f(+1px, -2, 5%) 1', 2),
                u'0 f()0': (u'0 f() 0', 3),
                u'f()0': (u'f() 0', 2),
                u'f()1%': (u'f() 1%', 2),
                u'f()1px': (u'f() 1px', 2),
                u'f()"str"': (u'f() "str"', 2),
                u'f()ident': (u'f() ident', 2),
                u'f()#123': (u'f() #123', 2),
                u'f()url()': (u'f() url()', 2),
                u'f()f()': (u'f() f()', 2),
                u'url(x.gif)0 0': (u'url(x.gif) 0 0', 3),
                u'url(x.gif)no-repeat': (u'url(x.gif) no-repeat', 2)
                }
        for (cssText, (c, l)) in tests.items():
            if c is None:
                c = cssText
            pv = cssutils.css.PropertyValue(cssText)
            self.assertEqual(c, pv.cssText)
            self.assertEqual(l, pv.length)

        tests = {
            # hash and rgb/a
            u'#112234': u'#112234',
            u'#112233': u'#123',
            u'rgb(1,2,3)': u'rgb(1, 2, 3)',
            u'rgb(  1  ,  2  ,  3  )': u'rgb(1, 2, 3)',
            u'rgba(1,2,3,4)': u'rgba(1, 2, 3, 4)',
            u'rgba(  1  ,  2  ,  3  ,  4 )': u'rgba(1, 2, 3, 4)',
            u'rgb(-1,+2,0)': u'rgb(-1, +2, 0)',
            u'rgba(-1,+2,0, 0)': u'rgba(-1, +2, 0, 0)',

            # FUNCTION
            u'f(1,2)': u'f(1, 2)',
            u'f(  1  ,  2  )': u'f(1, 2)',
            u'f(-1,+2)': u'f(-1, +2)',
            u'f(  -1  ,  +2  )': u'f(-1, +2)',
            u'fun(  -1  ,  +2  )': u'fun(-1, +2)',
            u'local( x )': u'local(x)',
            u'test(1px, #111, y, 1, 1%, "1", y(), var(x))':
                u'test(1px, #111, y, 1, 1%, "1", y(), var(x))',
            u'test(-1px, #111, y, -1, -1%, "1", -y())':
                u'test(-1px, #111, y, -1, -1%, "1", -y())',
            u'url(y)  format( "x" ,  "y" )': u'url(y) format("x", "y")',
            u'f(1 2,3 4)': u'f(1 2, 3 4)',

            # IE expression
            ur'Expression()': u'Expression()',
            ur'expression(-1 < +2)': u'expression(-1<+2)',
            ur'expression(document.width == "1")': u'expression(document.width=="1")',
            u'alpha(opacity=80)': u'alpha(opacity=80)',
            u'alpha( opacity = 80 , x=2  )': u'alpha(opacity=80, x=2)',
            u'expression(eval(document.documentElement.scrollTop))':
                u'expression(eval(document.documentElement.scrollTop))',
            #TODO
#            u'expression((function(ele){ele.style.behavior="none";})(this))':
#                u'expression((function(ele){ele.style.behavior="none";})(this))',

            # unicode-range
            'u+f': 'u+f',
            'U+ABCdef': 'u+abcdef',

            # url
            'url(a)': 'url(a)',
            'uRl(a)': 'url(a)',
            'u\\rl(a)': 'url(a)',
            'url("a")': 'url(a)',
            'url(  "a"  )': 'url(a)',
            'url(a)': 'url(a)',
            'url(";")': 'url(";")',
            'url(",")': 'url(",")',
            'url(")")': 'url(")")',
            '''url("'")''': '''url("'")''',
            '''url('"')''': '''url("\\"")''',
            '''url("'")''': '''url("'")''',

            # operator
            '1': '1',
            '1 2': '1 2',
            '1   2': '1 2',
            '1,2': '1, 2',
            '1,  2': '1, 2',
            '1  ,2': '1, 2',
            '1  ,  2': '1, 2',
            '1/2': '1/2',
            '1/  2': '1/2',
            '1  /2': '1/2',
            '1  /  2': '1/2',
             # comment
            '1/**/2': '1 /**/ 2',
            '1 /**/2': '1 /**/ 2',
            '1/**/ 2': '1 /**/ 2',
            '1 /**/ 2': '1 /**/ 2',
            '1  /*a*/  /*b*/  2': '1 /*a*/ /*b*/ 2',
            # , before
            '1,/**/2': '1, /**/ 2',
            '1 ,/**/2': '1, /**/ 2',
            '1, /**/2': '1, /**/ 2',
            '1 , /**/2': '1, /**/ 2',
            # , after
            '1/**/,2': '1 /**/, 2',
            '1/**/ ,2': '1 /**/, 2',
            '1/**/, 2': '1 /**/, 2',
            '1/**/ , 2': '1 /**/, 2',
            # all
            '1/*a*/  ,/*b*/  2': '1 /*a*/, /*b*/ 2',
            '1  /*a*/,  /*b*/2': '1 /*a*/, /*b*/ 2',
            '1  /*a*/  ,  /*b*/  2': '1 /*a*/, /*b*/ 2',

            # list
            'a b1,b2 b2,b3,b4': 'a b1, b2 b2, b3, b4',
            'a b1  ,   b2   b2  ,  b3  ,   b4': 'a b1, b2 b2, b3, b4',
            'u+1  ,   u+2-5': 'u+1, u+2-5',
            u'local( x ),  url(y)  format( "x" ,  "y" )':
                u'local(x), url(y) format("x", "y")',
            # FUNCTION
            u'attr( href )': u'attr(href)',
            # PrinceXML extende FUNC syntax with nested FUNC
            u'target-counter(attr(href),page)': u'target-counter(attr(href), page)'
                 }
        self.do_equal_r(tests)

        tests = [
                    u'a+',
                    u'-',
                    u'+',
                    u'-%',
                    u'+a',
                    u'--1px',
                    u'++1px',
                    u'#',
                    u'#00',
                    u'#12x',
                    u'#xyz',
                    u'#0000',
                    u'#00000',
                    u'#0000000',
                    u'-#0',
                    # operator
                    u',',
                    u'1,,2',
                    u'1,/**/,2',
                    u'1  ,  /**/  ,  2',
                    u'1,',
                    u'1, ',
                    u'1 ,',
                    u'1 , ',
                    u'1  ,  ',
                    u'1//2',
                    # URL
                    u'url(x))',
                    # string
                    u'"',
                    u"'",
                    # function
                    u'f(-)',
                    u'f(x))',
                    #calc
                    u'calc(',
                    u'calc(1',
                    u'calc(1 + 1',
                    u'calc(1+1)',
                    u'calc(1-1)',
                    u'calc(1 +1)',
                    u'calc(1+ 1)',
                    u'calc(1 -1)',
                    u'calc(1- 1)',
                    u'calc(+)',
                    u'calc(+ 1)',
                    u'calc(-)',
                    u'calc(- 1)',
                    u'calc(*)',
                    u'calc(*1)',
                    u'calc(* 2)',
                    u'calc(/)',
                    u'calc(/1)',
                    u'calc(/ 2)',
                    u'calc(1+)',
                    u'calc(1 +)',
                    u'calc(1 + )',
                    u'calc(2px -)',
                    u'calc(3px*)',
                    u'calc(3px *)',
                    u'calc(3px * )',
                    u'calc(4em/)',
                    u'calc(4em /)',
                    u'calc(4em / )',
                    u'calc(1 + + 1)',
                    u'calc(1 ++ 1)'
                    ]
        self.do_raise_r_list(tests, xml.dom.SyntaxErr)

    def test_list(self):
        "PropertyValue[index]"
        # issue #41
        css = """div.one {color: rgb(255, 0, 0);}   """
        sheet = cssutils.parseString(css)
        pv = sheet.cssRules[0].style.getProperty('color').propertyValue
        self.assertEqual(pv.value, 'rgb(255, 0, 0)')
        self.assertEqual(pv[0].value, 'rgb(255, 0, 0)')

        # issue #42
        sheet = cssutils.parseString('body { font-family: "A", b, serif }')
        pv = sheet.cssRules[0].style.getProperty('font-family').propertyValue
        self.assertEqual(3, pv.length)
        self.assertEqual(pv[0].value, 'A')
        self.assertEqual(pv[1].value, 'b')
        self.assertEqual(pv[2].value, 'serif')

    def test_comments(self):
        "PropertyValue with comment"
        # issue #45
        for t in (u'green',
                  u'green /* comment */',
                  u'/* comment */green',
                  u'/* comment */green/* comment */',
                  u'/* comment */  green  /* comment */',
                  u'/* comment *//**/  green  /* comment *//**/',
                  ):
            sheet = cssutils.parseString('body {color: %s; }' % t)
            p = sheet.cssRules[0].style.getProperties()[0]
            self.assertEqual(p.valid, True)
        for t in (u'gree',
                  u'gree /* comment */',
                  u'/* comment */gree',
                  u'/* comment */gree/* comment */',
                  u'/* comment */  gree  /* comment */',
                  u'/* comment *//**/  gree  /* comment *//**/',
                  ):
            sheet = cssutils.parseString('body {color: %s; }' % t)
            p = sheet.cssRules[0].style.getProperties()[0]
            self.assertEqual(p.valid, False)

    def test_incomplete(self):
        "PropertyValue (incomplete)"
        tests = {
            u'url("a': u'url(a)',
            u'url(a': u'url(a)'
        }
        for v, exp in tests.items():
            s = cssutils.parseString('a { background: %s' % v)
            v = s.cssRules[0].style.background
            self.assertEqual(v, exp)

    def test_readonly(self):
        "PropertyValue._readonly"
        v = cssutils.css.PropertyValue(cssText='inherit')
        self.assertTrue(False is v._readonly)

        v = cssutils.css.PropertyValue(cssText='inherit', readonly=True)
        self.assertTrue(True is v._readonly)
        self.assertTrue(u'inherit', v.cssText)
        self.assertRaises(xml.dom.NoModificationAllowedErr, v._setCssText, u'x')
        self.assertTrue(u'inherit', v.cssText)

    def test_reprANDstr(self):
        "PropertyValue.__repr__(), .__str__()"
        cssText='inherit'

        s = cssutils.css.PropertyValue(cssText=cssText)

        self.assertTrue(cssText in str(s))

        s2 = eval(repr(s))
        self.assertTrue(isinstance(s2, s.__class__))
        self.assertTrue(cssText == s2.cssText)


class ValueTestCase(basetest.BaseTestCase):

    def test_init(self):
        "Value.__init__()"
        v = cssutils.css.Value()
        self.assertTrue(u'' == v.cssText)
        self.assertTrue(u'' == v.value)
        self.assertTrue(None is  v.type)

    def test_cssText(self):
        "Value.cssText"
        # HASH IDENT STRING UNICODE-RANGE
        tests = {
                 u'#123': (u'#123', u'#123', u'HASH'),
                 u'#123456': (u'#123456', u'#123456', u'HASH'),
                 u'#112233': (u'#123', u'#112233', u'HASH'),
                 u'  #112233  ': (u'#123', u'#112233', u'HASH'),

                 u'red': (u'red', u'red', u'IDENT'),
                 u'  red  ': (u'red', u'red', u'IDENT'),
                 u'red  ': (u'red', u'red', u'IDENT'),
                 u'  red': (u'red', u'red', u'IDENT'),
                 u'red-': (u'red-', u'red-', u'IDENT'),
                 u'-red': (u'-red', u'-red', u'IDENT'),

                 u'"red"': (u'"red"', u'red', u'STRING'),
                 u"'red'": (u'"red"', u'red', u'STRING'),
                 u'  "red"  ': (u'"red"', u'red', u'STRING'),
                 ur'"red\""': (ur'"red\""', ur'red"', u'STRING'),
                 ur"'x\"'": (ur'"x\\""', ur'x\"', 'STRING'), #???
                 u'''"x\
y"''': (u'"xy"', u'xy', u'STRING'),
                 }
        for (p, (r, n, t)) in tests.items():
            v = cssutils.css.Value(p)
            self.assertEqual(r, v.cssText)
            self.assertEqual(t, v.type)
            self.assertEqual(n, v.value)


class ColorValueTestCase(basetest.BaseTestCase):

    def test_init(self):
        "ColorValue.__init__()"
        v = cssutils.css.ColorValue()
        self.assertEqual(v.COLOR_VALUE, v.type)
        self.assertTrue(u'' == v.cssText)
        self.assertTrue(u'' == v.value)
        self.assertEqual(u'transparent', v.name)
        self.assertEqual(None, v.colorType)

    def test_cssText(self):
        "ColorValue.cssText"
        tests = {
                 # HASH
                 u'#123': (u'#123',),
                 u'#112233': (u'#123',),
                 # rgb
                 u'rgb(1,2,3)': (u'rgb(1, 2, 3)',),
                 u'rgb(1%,2%,3%)': (u'rgb(1%, 2%, 3%)',),
                 u'rgb(-1,-1,-1)': (u'rgb(-1, -1, -1)',),
                 u'rgb(-1%,-2%,-3%)': (u'rgb(-1%, -2%, -3%)',),
                 # rgba
                 u'rgba(1,2,3, 0)': (u'rgba(1, 2, 3, 0)',),
                 # hsl
                 u'hsl(1,2%,3%)': (u'hsl(1, 2%, 3%)',),
                 u'hsla(1,2%,3%, 1.0)': (u'hsla(1, 2%, 3%, 1)',),

                 }
        for (p, (r, )) in tests.items():
            v = cssutils.css.ColorValue(p)
            self.assertEqual(v.COLOR_VALUE, v.type)
            self.assertEqual(r, v.cssText)
            self.assertEqual(r, v.value)

            v = cssutils.css.ColorValue()
            v.cssText = p
            self.assertEqual(v.COLOR_VALUE, v.type)
            self.assertEqual(r, v.cssText)
            self.assertEqual(r, v.value)

        tests = {
             u'1': xml.dom.SyntaxErr,
             u'a': xml.dom.SyntaxErr,

             u'#12': xml.dom.SyntaxErr,
             u'#1234': xml.dom.SyntaxErr,
             u'#1234567': xml.dom.SyntaxErr,
             u'#12345678': xml.dom.SyntaxErr,

             u'rgb(1,1%,1%)': xml.dom.SyntaxErr,
             u'rgb(1%,1,1)': xml.dom.SyntaxErr,
             u'rgb(-1,-1%,-1%)': xml.dom.SyntaxErr,
             u'rgb(-1%,-1,-1)': xml.dom.SyntaxErr,

             u'rgb(1,1,1, 0)': xml.dom.SyntaxErr,
             u'rgb(1%,1%,1%, 0)': xml.dom.SyntaxErr,
             u'rgba(1,1,1)': xml.dom.SyntaxErr,
             u'rgba(1%,1%,1%)': xml.dom.SyntaxErr,
             u'rgba(1,1,1, 0%)': xml.dom.SyntaxErr,
             u'rgba(1%,1%,1%, 0%)': xml.dom.SyntaxErr,

             u'hsl(1,2%,3%, 1)': xml.dom.SyntaxErr,
             u'hsla(1,2%,3%)': xml.dom.SyntaxErr,

             u'hsl(1,2,3)': xml.dom.SyntaxErr,
             u'hsl(1%,2,3)': xml.dom.SyntaxErr,
             u'hsl(1%,2,3%)': xml.dom.SyntaxErr,
             u'hsl(1%,2%,3)': xml.dom.SyntaxErr,

             u'hsla(1,2%,3%, 0%)': xml.dom.SyntaxErr,
             u'hsla(1,2,3, 0.0)': xml.dom.SyntaxErr,
             u'hsla(1%,2,3, 0.0)': xml.dom.SyntaxErr,
             u'hsla(1%,2,3%, 0.0)': xml.dom.SyntaxErr,
             u'hsla(1%,2%,3, 0.0)': xml.dom.SyntaxErr,
        }
        self.r = cssutils.css.ColorValue()
        self.do_raise_r(tests)

    def test_rgb(self):
        "ColorValue.red .green .blue"
        tests = {
            (u'#0A0AD2', 'rgb(10, 10, 210)' ): (10, 10, 210, 1.0),
            # TODO: Fix rounding?
            (u'hsl(240, 91%, 43%)', ): (10, 10, 209, 1.0),
            (u'#ff8800', u'#f80',
             'rgb(255, 136, 0)', 'rgba(255, 136, 0, 1.0)'): (255, 136, 0, 1.0),
            (u'red', u'#ff0000', u'#f00',
             u'hsl(0, 100%, 50%)', u'hsla(0, 100%, 50%, 1.0)'):
                (255, 0, 0, 1.0),
            (u'lime', u'#00ff00', u'#0f0', u'hsl(120, 100%, 50%)'):
                (0, 255, 0, 1.0),
            (u'rgba(255, 127, 0, .1)', u'rgba(100%, 50%, 0%, .1)'):
                (255, 127, 0, 0.1),
            (u'transparent', u'rgba(0, 0, 0, 0)'): (0, 0, 0, 0),
            (u'aqua',): (0, 255, 255, 1.0)
        }
        for colors, rgba in tests.items():
            for color in colors:
                c = cssutils.css.ColorValue(color);
                self.assertEqual(c.red, rgba[0])
                self.assertEqual(c.green, rgba[1])
                self.assertEqual(c.blue, rgba[2])
                self.assertEqual(c.alpha, rgba[3])


class URIValueTestCase(basetest.BaseTestCase):

    def test_init(self):
        "URIValue.__init__()"
        v = cssutils.css.URIValue()
        self.assertTrue(u'url()' == v.cssText)
        self.assertTrue(u'' == v.value)
        self.assertTrue(u'' == v.uri)
        self.assertTrue(v.URI is  v.type)

        v.uri = '1'
        self.assertTrue(u'1' == v.value)
        self.assertTrue(u'1' == v.uri)
        self.assertEqual(u'url(1)', v.cssText)

        v.value = '2'
        self.assertTrue(u'2' == v.value)
        self.assertTrue(u'2' == v.uri)
        self.assertEqual(u'url(2)', v.cssText)

    def test_absoluteUri(self):
        "URIValue.absoluteUri"
        s = cssutils.parseString('a { background-image: url(x.gif)}', href="/path/to/x.css")
        v = s.cssRules[0].style.getProperty('background-image').propertyValue[0]
        self.assertEqual(u'x.gif', v.uri)
        self.assertEqual(u'/path/to/x.gif', v.absoluteUri)

        v = cssutils.css.URIValue(u'url(x.gif)')
        self.assertEqual(u'x.gif', v.uri)
        self.assertEqual(u'x.gif', v.absoluteUri)

    def test_cssText(self):
        "URIValue.cssText"
        tests = {
                 u'url()': (u'url()', u'', u'URI'),
                 # comments are part of the url!
                 u'url(/**/)': (u'url(/**/)', u'/**/', u'URI'),
                 u'url(/**/1)': (u'url(/**/1)', u'/**/1', u'URI'),
                 u'url(1/**/)': (u'url(1/**/)', u'1/**/', u'URI'),
                 u'url(/**/1/**/)': (u'url(/**/1/**/)', u'/**/1/**/', u'URI'),
                 u'url(some.gif)': (u'url(some.gif)', u'some.gif', u'URI'),
                 u'  url(some.gif)  ': (u'url(some.gif)', u'some.gif', u'URI'),
                 u'url(   some.gif  )': (u'url(some.gif)', u'some.gif', u'URI'),
                 }
        for (p, (r, n, t)) in tests.items():
            v = cssutils.css.URIValue(p)
            self.assertEqual(r, v.cssText)
            self.assertEqual(t, v.type)
            self.assertEqual(n, v.value)
            self.assertEqual(n, v.uri)

            v = cssutils.css.URIValue()
            v.cssText = p
            self.assertEqual(r, v.cssText)
            self.assertEqual(t, v.type)
            self.assertEqual(n, v.value)
            self.assertEqual(n, v.uri)

        tests = {
             u'a()': xml.dom.SyntaxErr,
             u'1': xml.dom.SyntaxErr,
             u'url(': xml.dom.SyntaxErr,
             u'url("': xml.dom.SyntaxErr,
             u'url(\'': xml.dom.SyntaxErr,
             }
        self.r = cssutils.css.URIValue()
        self.do_raise_r(tests)

class DimensionValueTestCase(basetest.BaseTestCase):

    def test_init(self):
        "DimensionValue.__init__()"
        v = cssutils.css.DimensionValue()
        self.assertTrue(u'' == v.cssText)
        self.assertTrue(u'' == v.value)
        self.assertTrue(None is  v.type)
        self.assertTrue(None is  v.dimension)

    def test_cssText(self):
        "DimensionValue.cssText"
        # NUMBER DIMENSION PERCENTAGE
        tests = {

                 u'0': (u'0', 0, None, u'NUMBER'),
                 u'00': (u'0', 0, None, u'NUMBER'),
                 u'.0': (u'0', 0, None, u'NUMBER'),
                 u'0.0': (u'0', 0, None, u'NUMBER'),
                 u'+0': (u'0', 0, None, u'NUMBER'),
                 u'+00': (u'0', 0, None, u'NUMBER'),
                 u'+.0': (u'0', 0, None, u'NUMBER'),
                 u'+0.0': (u'0', 0, None, u'NUMBER'),
                 u'-0': (u'0', 0, None, u'NUMBER'),
                 u'-00': (u'0', 0, None, u'NUMBER'),
                 u'-.0': (u'0', 0, None, u'NUMBER'),
                 u'-0.0': (u'0', 0, None, u'NUMBER'),

                 u'1': (u'1', 1, None, u'NUMBER'),
                 u'1.0': (u'1', 1.0, None, u'NUMBER'),
                 u'1.1': (u'1.1', 1.1, None, u'NUMBER'),
                 u'+1': (u'+1', 1, None, u'NUMBER'),
                 u'+1.0': (u'+1', 1.0, None, u'NUMBER'),
                 u'+1.1': (u'+1.1', 1.1, None, u'NUMBER'),
                 u'-1': (u'-1', -1, None, u'NUMBER'),
                 u'-1.0': (u'-1', -1, None, u'NUMBER'),
                 u'-1.1': (u'-1.1', -1.1, None, u'NUMBER'),

                 u'0px': (u'0', 0, u'px', u'DIMENSION'),
                 u'1px': (u'1px', 1, u'px', u'DIMENSION'),
                 u'1.0px': (u'1px', 1.0, u'px', u'DIMENSION'),
                 u'1.1px': (u'1.1px', 1.1, u'px', u'DIMENSION'),
                 u'-1px': (u'-1px', -1, u'px', u'DIMENSION'),
                 u'-1.1px': (u'-1.1px', -1.1, u'px', u'DIMENSION'),
                 u'+1px': (u'+1px', 1, u'px', u'DIMENSION'),

                 u'1px1': (u'1px1', 1, u'px1', u'DIMENSION'),

                 u'0%': (u'0%', 0, u'%', u'PERCENTAGE'),
                 u'1%': (u'1%', 1, u'%', u'PERCENTAGE'),
                 u'1.1%': (u'1.1%', 1.1, u'%', u'PERCENTAGE'),
                 u'-1%': (u'-1%', -1, u'%', u'PERCENTAGE'),
                 u'-1.1%': (u'-1.1%', -1.1, u'%', u'PERCENTAGE'),
                 u'+1%': (u'+1%', 1, u'%', u'PERCENTAGE'),
                 }
        for (p, (r, n, d, t)) in tests.items():
            v = cssutils.css.DimensionValue(p)
            self.assertEqual(r, v.cssText)
            self.assertEqual(t, v.type)
            self.assertEqual(n, v.value)
            self.assertEqual(d, v.dimension)


class CSSFunctionTestCase(basetest.BaseTestCase):

    def test_init(self):
        "CSSFunction.__init__()"
        v = cssutils.css.CSSFunction()
        self.assertEqual(u'', v.cssText)
        self.assertEqual(u'FUNCTION', v.type)
        self.assertEqual(v.value, u'')

    def test_cssText(self):
        "CSSFunction.cssText"
        tests = {
                 u'x(x)': (u'x(x)', None),
                 u'X(  X  )': (u'x(X)', None),
                 u'x(1,2)': (u'x(1, 2)', None),
                 u'x(1/**/)': (u'x(1 /**/)', u'x(1)'),
                 u'x(/**/1)': (u'x(/**/ 1)', u'x(1)'),
                 u'x(/**/1/**/)': (u'x(/**/ 1 /**/)', u'x(1)'),
                 u'x(/**/1,x/**/)': (u'x(/**/ 1, x /**/)', u'x(1, x)'),
                 u'x(1,2)': (u'x(1, 2)', None),
                 }
        for (f, (cssText, value)) in tests.items():
            if value is None:
                value = cssText
            v = cssutils.css.CSSFunction(f)
            self.assertEqual(cssText, v.cssText)
            self.assertEqual('FUNCTION', v.type)
            self.assertEqual(value, v.value)


class CSSVariableTestCase(basetest.BaseTestCase):

    def test_init(self):
        "CSSVariable.__init__()"
        v = cssutils.css.CSSVariable()
        self.assertEqual(u'', v.cssText)
        self.assertEqual(u'VARIABLE', v.type)
        self.assertTrue(None is v.name)
        self.assertTrue(None is v.value)

    def test_cssText(self):
        "CSSVariable.cssText"
        tests = {
                 u'var(x)': (u'var(x)', u'x', None),
                 u'VAR(  X  )': (u'var(X)', u'X', None),
                 u'var(c1,rgb(14,14,14))': (u'var(c1, rgb(14, 14, 14))', u'c1', u'rgb(14, 14, 14)'),
                 u'var( L, 1px )': (u'var(L, 1px)', u'L', u'1px'),
                 u'var(L,1)': (u'var(L, 1)', u'L', u'1'),
                 u'var(T, calc( 2 * 1px ))': (u'var(T, calc(2 * 1px))', u'T', 'calc(2 * 1px)'),
                 u'var(U, url( example.png ) )': (u'var(U, url(example.png))', u'U', u'url(example.png)'),
                 u'var(C, #f00 )': (u'var(C, #f00)', u'C', '#fff')
                 }
        for (var, (cssText, name, fallback)) in tests.items():
            v = cssutils.css.CSSVariable(var)
            self.assertEqual(cssText, v.cssText)
            self.assertEqual('VARIABLE', v.type)
            self.assertEqual(name, v.name)
            # not resolved so it is None
            self.assertEqual(None, v.value)



#    def test_cssValueType(self):
#        "CSSValue.cssValueType .cssValueTypeString"
#        tests = [
#            ([u'inherit', u'INhe\\rit'], 'CSS_INHERIT', cssutils.css.CSSValue),
#            (['1', '1%', '1em', '1ex', '1px', '1cm', '1mm', '1in', '1pt', '1pc',
#              '1deg', '1rad', '1grad', '1ms', '1s', '1hz', '1khz', '1other',
#               '"string"', "'string'", 'url(x)', 'red',
#               'attr(a)', 'counter(x)', 'rect(1px, 2px, 3px, 4px)',
#               'rgb(0, 0, 0)', '#000', '#123456', 'rgba(0, 0, 0, 0)',
#               'hsl(0, 0, 0)', 'hsla(0, 0, 0, 0)',
#               ],
#             'CSS_PRIMITIVE_VALUE', cssutils.css.CSSPrimitiveValue),
#            ([u'1px 1px', 'red blue green x'], 'CSS_VALUE_LIST', cssutils.css.CSSValueList),
#            # what is a custom value?
#            #([], 'CSS_CUSTOM', cssutils.css.CSSValue)
#            ]
#        for values, name, cls in tests:
#            for value in values:
#                v = cssutils.css.CSSValue(cssText=value)
#                if value == "'string'":
#                    # will be changed to " always
#                    value = '"string"'
#                self.assertEqual(value, v.cssText)
#                self.assertEqual(name, v.cssValueTypeString)
#                self.assertEqual(getattr(v, name), v.cssValueType)
#                self.assertEqual(cls, type(v))


#class CSSPrimitiveValueTestCase(basetest.BaseTestCase):
#
#    def test_init(self):
#        "CSSPrimitiveValue.__init__()"
#        v = cssutils.css.CSSPrimitiveValue(u'1')
#        self.assertTrue(u'1' == v.cssText)
#
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue("CSS_PRIMITIVE_VALUE" == v.cssValueTypeString)
#
#        self.assertTrue(v.CSS_NUMBER == v.primitiveType)
#        self.assertTrue("CSS_NUMBER" == v.primitiveTypeString)
#
#        # DUMMY to be able to test empty constructor call
#        #self.assertRaises(xml.dom.SyntaxErr, v.__init__, None)
#
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getCounterValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getRGBColorValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getRectValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getStringValue)
#
#    def test_CSS_UNKNOWN(self):
#        "CSSPrimitiveValue.CSS_UNKNOWN"
#        v = cssutils.css.CSSPrimitiveValue(u'expression(false)')
#        self.assertTrue(v.CSS_UNKNOWN == v.primitiveType)
#        self.assertTrue('CSS_UNKNOWN' == v.primitiveTypeString)
#
#    def test_CSS_NUMBER_AND_OTHER_DIMENSIONS(self):
#        "CSSPrimitiveValue.CSS_NUMBER .. CSS_DIMENSION"
#        defs = [
#            ('', 'CSS_NUMBER'),
#            ('%', 'CSS_PERCENTAGE'),
#            ('em', 'CSS_EMS'),
#            ('ex', 'CSS_EXS'),
#            ('px', 'CSS_PX'),
#            ('cm', 'CSS_CM'),
#            ('mm', 'CSS_MM'),
#            ('in', 'CSS_IN'),
#            ('pt', 'CSS_PT'),
#            ('pc', 'CSS_PC'),
#            ('deg', 'CSS_DEG'),
#            ('rad', 'CSS_RAD'),
#            ('grad', 'CSS_GRAD'),
#            ('ms', 'CSS_MS'),
#            ('s', 'CSS_S'),
#            ('hz', 'CSS_HZ'),
#            ('khz', 'CSS_KHZ'),
#            ('other_dimension', 'CSS_DIMENSION')
#            ]
#        for dim, name in defs:
#            for n in (0, 1, 1.1, -1, -1.1, -0):
#                v = cssutils.css.CSSPrimitiveValue('%i%s' % (n, dim))
#                self.assertEqual(name, v.primitiveTypeString)
#                self.assertEqual(getattr(v, name), v.primitiveType)
#
#    def test_CSS_STRING_AND_OTHER(self):
#        "CSSPrimitiveValue.CSS_STRING .. CSS_RGBCOLOR"
#        defs = [
#                (('""', "''", '"some thing"', "' A\\ND '",
#                  # comma separated lists are STRINGS FOR NOW!
#                  'a, b',
#                  '"a", "b"',
#                  ), 'CSS_STRING'),
#                (('url(a)', 'url("a b")', "url(' ')"), 'CSS_URI'),
#                (('some', 'or_anth-er'), 'CSS_IDENT'),
#                (('attr(a)', 'attr(b)'), 'CSS_ATTR'),
#                (('counter(1)', 'counter(2)'), 'CSS_COUNTER'),
#                (('rect(1,2,3,4)',), 'CSS_RECT'),
#                (('rgb(1,2,3)', 'rgb(10%, 20%, 30%)', '#123', '#123456'),
#                 'CSS_RGBCOLOR'),
#                (('rgba(1,2,3,4)','rgba(10%, 20%, 30%, 40%)', ),
#                 'CSS_RGBACOLOR'),
#                (('U+0', 'u+ffffff', 'u+000000-f',
#                  'u+0-f, U+ee-ff'), 'CSS_UNICODE_RANGE')
#                ]
#
#        for examples, name in defs:
#            for x in examples:
#                v = cssutils.css.CSSPrimitiveValue(x)
#                self.assertEqual(getattr(v, name), v.primitiveType)
#                self.assertEqual(name, v.primitiveTypeString)
#
#    def test_getFloat(self):
#        "CSSPrimitiveValue.getFloatValue()"
#        # NOT TESTED are float values as it seems difficult to
#        # compare these. Maybe use decimal.Decimal?
#
#        v = cssutils.css.CSSPrimitiveValue(u'1px')
#        tests = {
#            '0': (v.CSS_NUMBER, 0),
#            '-1.1': (v.CSS_NUMBER, -1.1),
#            '1%': (v.CSS_PERCENTAGE, 1),
#            '-1%': (v.CSS_PERCENTAGE, -1),
#            '1em': (v.CSS_EMS, 1),
#            '-1.1em': (v.CSS_EMS, -1.1),
#            '1ex': (v.CSS_EXS, 1),
#            '1px': (v.CSS_PX, 1),
#
#            '1cm': (v.CSS_CM, 1),
#            '1cm': (v.CSS_MM, 10),
#            '254cm': (v.CSS_IN, 100),
#            '1mm': (v.CSS_MM, 1),
#            '10mm': (v.CSS_CM, 1),
#            '254mm': (v.CSS_IN, 10),
#            '1in': (v.CSS_IN, 1),
#            '100in': (v.CSS_CM, 254), # ROUNDED!!!
#            '10in': (v.CSS_MM, 254), # ROUNDED!!!
#
#            '1pt': (v.CSS_PT, 1),
#            '1pc': (v.CSS_PC, 1),
#
#            '1deg': (v.CSS_DEG, 1),
#            '1rad': (v.CSS_RAD, 1),
#            '1grad': (v.CSS_GRAD, 1),
#
#            '1ms': (v.CSS_MS, 1),
#            '1000ms': (v.CSS_S, 1),
#            '1s': (v.CSS_S, 1),
#            '1s': (v.CSS_MS, 1000),
#
#            '1hz': (v.CSS_HZ, 1),
#            '1000hz': (v.CSS_KHZ, 1),
#            '1khz': (v.CSS_KHZ, 1),
#            '1khz': (v.CSS_HZ, 1000),
#
#            '1DIMENSION': (v.CSS_DIMENSION, 1),
#            }
#        for cssText in tests:
#            v.cssText = cssText
#            unitType, exp = tests[cssText]
#            val = v.getFloatValue(unitType)
#            if unitType in (v.CSS_IN, v.CSS_CM):
#                val = round(val)
#            self.assertEqual(val , exp)
#
#    def test_setFloat(self):
#        "CSSPrimitiveValue.setFloatValue()"
#        V = cssutils.css.CSSPrimitiveValue
#
#        tests = {
#            # unitType, value
#            (V.CSS_NUMBER, 1): [
#                # unitType, setvalue,
#                #    getvalue or expected exception, msg or cssText
#                (V.CSS_NUMBER, 0, 0, '0'),
#                (V.CSS_NUMBER, 0.1, 0.1, '0.1'),
#                (V.CSS_NUMBER, -0, 0, '0'),
#                (V.CSS_NUMBER, 2, 2, '2'),
#                (V.CSS_NUMBER, 2.0, 2, '2'),
#                (V.CSS_NUMBER, 2.1, 2.1, '2.1'),
#                (V.CSS_NUMBER, -2.1, -2.1, '-2.1'),
#                # setting with string does work
#                (V.CSS_NUMBER, '1', 1, '1'),
#                (V.CSS_NUMBER, '1.1', 1.1, '1.1'),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_RAD, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_GRAD, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_S, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_MS, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_KHZ, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_HZ, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DIMENSION, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_MM, 2, xml.dom.InvalidAccessErr, None),
#
#                (V.CSS_NUMBER, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: floatValue 'x' is not a float"),
#                (V.CSS_NUMBER, '1x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: floatValue '1x' is not a float"),
#
#                (V.CSS_STRING, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_STRING' is not a float type"),
#                (V.CSS_URI, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_URI' is not a float type"),
#                (V.CSS_ATTR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_ATTR' is not a float type"),
#                (V.CSS_IDENT, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_IDENT' is not a float type"),
#                (V.CSS_RGBCOLOR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RGBCOLOR' is not a float type"),
#                (V.CSS_RGBACOLOR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RGBACOLOR' is not a float type"),
#                (V.CSS_RECT, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RECT' is not a float type"),
#                (V.CSS_COUNTER, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_COUNTER' is not a float type"),
#                (V.CSS_EMS, 1, xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_NUMBER' to 'CSS_EMS'"),
#                (V.CSS_EXS, 1, xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_NUMBER' to 'CSS_EXS'")
#            ],
#            (V.CSS_MM, '1mm'): [
#                (V.CSS_MM, 2, 2, '2mm'),
#                (V.CSS_MM, 0, 0, '0mm'),
#                (V.CSS_MM, 0.1, 0.1, '0.1mm'),
#                (V.CSS_MM, -0, -0, '0mm'),
#                (V.CSS_MM, 3.0, 3, '3mm'),
#                (V.CSS_MM, 3.1, 3.1, '3.1mm'),
#                (V.CSS_MM, -3.1, -3.1, '-3.1mm'),
#                (V.CSS_CM, 1, 10, '10mm'),
#                (V.CSS_IN, 10, 254, '254mm'),
#                (V.CSS_PT, 1, 1828.8, '1828.8mm'),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_NUMBER, 2, xml.dom.InvalidAccessErr, None)
#            ],
#            (V.CSS_PT, '1pt'): [
#                (V.CSS_PT, 2, 2, '2pt'),
#                (V.CSS_PC, 12, 1, '1pt'),
#                (V.CSS_NUMBER, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None)
#            ],
#            (V.CSS_KHZ, '1khz'): [
#                (V.CSS_HZ, 2000, 2, '2khz'),
#                (V.CSS_NUMBER, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None)
#            ]
#        }
#        for test in tests:
#            initialType, initialValue = test
#            pv = cssutils.css.CSSPrimitiveValue(initialValue)
#            for setType, setValue, exp, cssText in tests[test]:
#                if type(exp) == types.TypeType or\
#                   type(exp) == types.ClassType: # 2.4 compatibility
#                    if cssText:
#                        self.assertRaisesMsg(
#                            exp, cssText, pv.setFloatValue, setType, setValue)
#                    else:
#                        self.assertRaises(
#                            exp, pv.setFloatValue, setType, setValue)
#                else:
#                    pv.setFloatValue(setType, setValue)
#                    self.assertEqual(pv._value[0], cssText)
#                    if cssText == '0mm':
#                        cssText = '0'
#                    self.assertEqual(pv.cssText, cssText)
#                    self.assertEqual(pv.getFloatValue(initialType), exp)
#
#    def test_getString(self):
#        "CSSPrimitiveValue.getStringValue()"
#        v = cssutils.css.CSSPrimitiveValue(u'1px')
#        self.assertTrue(v.primitiveType == v.CSS_PX)
#        self.assertRaises(xml.dom.InvalidAccessErr,
#                          v.getStringValue)
#
#        pv = cssutils.css.CSSPrimitiveValue
#        tests = {
#            pv.CSS_STRING: ("'red'", 'red'),
#            pv.CSS_STRING: ('"red"', 'red'),
#            pv.CSS_URI: ('url(http://example.com)', None),
#            pv.CSS_URI: ("url('http://example.com')",
#                         u"http://example.com"),
#            pv.CSS_URI: ('url("http://example.com")',
#                         u'http://example.com'),
#            pv.CSS_URI: ('url("http://example.com?)")',
#                         u'http://example.com?)'),
#            pv.CSS_IDENT: ('red', None),
#            pv.CSS_ATTR: ('attr(att-name)',
#                         u'att-name'), # the name of the attrr
#            }
#        for t in tests:
#            val, exp = tests[t]
#            if not exp:
#                exp = val
#
#            v = cssutils.css.CSSPrimitiveValue(val)
#            self.assertEqual(v.primitiveType, t)
#            self.assertEqual(v.getStringValue(), exp)
#
#    def test_setString(self):
#        "CSSPrimitiveValue.setStringValue()"
#        # CSS_STRING
#        v = cssutils.css.CSSPrimitiveValue(u'"a"')
#        self.assertTrue(v.CSS_STRING == v.primitiveType)
#        v.setStringValue(v.CSS_STRING, 'b')
#        self.assertTrue(('b', 'STRING') == v._value)
#        self.assertEqual('b', v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_IDENT
#        v = cssutils.css.CSSPrimitiveValue('new')
#        v.setStringValue(v.CSS_IDENT, 'ident')
#        self.assertTrue(v.CSS_IDENT == v.primitiveType)
#        self.assertTrue(('ident', 'IDENT') == v._value)
#        self.assertTrue('ident' == v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_URI
#        v = cssutils.css.CSSPrimitiveValue('url(old)')
#        v.setStringValue(v.CSS_URI, '(')
#        self.assertEqual((u'(', 'URI'), v._value)
#        self.assertEqual(u'(', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, ')')
#        self.assertEqual((u')', 'URI'), v._value)
#        self.assertEqual(u')', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, '"')
#        self.assertEqual(ur'"', v.getStringValue())
#        self.assertEqual((ur'"', 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, "''")
#        self.assertEqual(ur"''", v.getStringValue())
#        self.assertEqual((ur"''", 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, ',')
#        self.assertEqual(ur',', v.getStringValue())
#        self.assertEqual((ur',', 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, ' ')
#        self.assertEqual((u' ', 'URI'), v._value)
#        self.assertEqual(u' ', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, 'a)')
#        self.assertEqual((u'a)', 'URI'), v._value)
#        self.assertEqual(u'a)', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, 'a')
#        self.assertTrue(v.CSS_URI == v.primitiveType)
#        self.assertEqual((u'a', 'URI'), v._value)
#        self.assertEqual(u'a', v.getStringValue())
#
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_ATTR
#        v = cssutils.css.CSSPrimitiveValue('attr(old)')
#        v.setStringValue(v.CSS_ATTR, 'a')
#        self.assertTrue(v.CSS_ATTR == v.primitiveType)
#        self.assertTrue('a' == v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#
#        # TypeError as 'x' is no valid type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType 'x' (UNKNOWN TYPE) is not a string type",
#            v.setStringValue, *('x', 'brown'))
#        # IndexError as 111 is no valid type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType 111 (UNKNOWN TYPE) is not a string type",
#            v.setStringValue, *(111, 'brown'))
#        # CSS_PX is no string type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType CSS_PX is not a string type",
#            v.setStringValue, *(v.CSS_PX, 'brown'))
#
#    def test_typeRGBColor(self):
#        "RGBColor"
#        v = cssutils.css.CSSPrimitiveValue('RGB(1, 5, 10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1, 5, 10)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('rgb(1, 5, 10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1, 5, 10)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('rgb(1%, 5%, 10%)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1%, 5%, 10%)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('  rgb(  1 ,5,  10  )')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        v = cssutils.css.CSSPrimitiveValue('rgb(1,5,10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        v = cssutils.css.CSSPrimitiveValue('rgb(1%, .5%, 10.1%)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)


if __name__ == '__main__':
    import unittest
    unittest.main()



"""Experimental settings for special stuff."""

def set(key, value):
    """Call to enable special settings:
    
    ('DXImageTransform.Microsoft', True)
        enable support for parsing special MS only filter values
    
    Clears the tokenizer cache which holds the compiled productions!
    """
    if key == 'DXImageTransform.Microsoft' and value == True:
        import cssproductions
        import tokenize2
        tokenize2._TOKENIZER_CACHE.clear()
        cssproductions.PRODUCTIONS.insert(1, cssproductions._DXImageTransform)



"""Testcases for cssutils.settings"""
__version__ = '$Id: test_csscharsetrule.py 1356 2008-07-13 17:29:09Z cthedot $'

import test_cssrule
import cssutils
import cssutils.settings

class Settings(test_cssrule.CSSRuleTestCase):
        
    def test_set(self):
        "settings.set()"
        cssutils.ser.prefs.useMinified()
        text = u'a {filter: progid:DXImageTransform.Microsoft.BasicImage( rotation = 90 )}'
        
        self.assertEqual(cssutils.parseString(text).cssText, ''.encode())
        
        cssutils.settings.set('DXImageTransform.Microsoft', True)
        self.assertEqual(cssutils.parseString(text).cssText,
                         'a{filter:progid:DXImageTransform.Microsoft.BasicImage(rotation=90)}'.encode())

        cssutils.ser.prefs.useDefaults()
        

if __name__ == '__main__':
    import unittest
    unittest.main()



"""Testcases for cssutils.css.selectorlist.SelectorList."""

import xml.dom
import basetest
import cssutils
from cssutils.css.selectorlist import SelectorList

class SelectorListTestCase(basetest.BaseTestCase):

    def setUp(self):
        self.r = SelectorList()

    def test_init(self):
        "SelectorList.__init__() and .length"
        s = SelectorList()
        self.assertEqual(0, s.length)

        s = SelectorList('a, b')
        self.assertEqual(2, s.length)
        self.assertEqual(u'a, b', s.selectorText)

        s = SelectorList(selectorText='a')
        self.assertEqual(1, s.length)
        self.assertEqual(u'a', s.selectorText)

        s = SelectorList(selectorText=('p|a', {'p': 'uri'})) # n-dict
        self.assertEqual(1, s.length)
        self.assertEqual(u'p|a', s.selectorText)

        s = SelectorList(selectorText=('p|a', (('p', 'uri'),))) # n-tuples
        self.assertEqual(1, s.length)
        self.assertEqual(u'p|a', s.selectorText)

    def test_parentRule(self):
        "Selector.parentRule"
        def check(style):
            self.assertEqual(style, style.selectorList.parentRule)
            for sel in style.selectorList:
                self.assertEqual(style.selectorList, sel.parent)
        
        style = cssutils.css.CSSStyleRule('a, b')
        check(style)

        # add new selector
        style.selectorList.append(cssutils.css.Selector('x'))
        check(style)

        # replace selectorList
        style.selectorList = cssutils.css.SelectorList('x')
        check(style)

        # replace selectorText
        style.selectorText = ('x, y')
        check(style)
            
    def test_appendSelector(self):
        "SelectorList.appendSelector() and .length"
        s = SelectorList()
        s.appendSelector('a')
        self.assertEqual(1, s.length)

        self.assertRaises(xml.dom.InvalidModificationErr,
                          s.appendSelector, 'b,')
        self.assertEqual(1, s.length)

        self.assertEqual(u'a', s.selectorText)

        s.append('b')
        self.assertEqual(2, s.length)
        self.assertEqual(u'a, b', s.selectorText)

        s.append('a')
        self.assertEqual(2, s.length)
        self.assertEqual(u'b, a', s.selectorText)
        
        # __setitem__    
        self.assertRaises(IndexError, s.__setitem__, 4, 'x')
        s[1] = 'c'
        self.assertEqual(2, s.length)
        self.assertEqual(u'b, c', s.selectorText)
        # TODO: remove duplicates?
#        s[0] = 'c'
#        self.assertEqual(1, s.length)
#        self.assertEqual(u'c', s.selectorText)

        s = SelectorList()
        s.appendSelector(('p|a', {'p': 'uri', 'x': 'xxx'}))
        self.assertEqual(u'p|a', s.selectorText)
        # x gets lost as not used
        self.assertRaises(xml.dom.NamespaceErr, s.append, 'x|a')
        # not set at all
        self.assertRaises(xml.dom.NamespaceErr, s.append, 'y|a')
        # but p is retained
        s.append('p|b')
        self.assertEqual(u'p|a, p|b', s.selectorText)

    def test_selectorText(self):
        "SelectorList.selectorText"
        s = SelectorList()
        s.selectorText = u'a, b'
        self.assertEqual(u'a, b', s.selectorText)
        self.assertRaises(xml.dom.SyntaxErr, s._setSelectorText, u',')
        # not changed as invalid!
        self.assertEqual(u'a, b', s.selectorText)

        tests = {
            u'*': None,
            u'/*1*/*': None,
            u'/*1*/*, a': None,
            u'a, b': None,
            u'a ,b': u'a, b',
            u'a , b': u'a, b',
            u'a, b, c': u'a, b, c',
            u'#a, x#a, .b, x.b': u'#a, x#a, .b, x.b',
            (u'[p|a], p|*', (('p', 'uri'),)): u'[p|a], p|*',
            }
        # do not parse as not complete
        self.do_equal_r(tests, att='selectorText')

        tests = {
            u'x|*': xml.dom.NamespaceErr,
            u'': xml.dom.SyntaxErr,
            u' ': xml.dom.SyntaxErr,
            u',': xml.dom.SyntaxErr,
            u'a,': xml.dom.SyntaxErr,
            u',a': xml.dom.SyntaxErr,
            u'/* 1 */,a': xml.dom.SyntaxErr,
            }
        # only set as not complete
        self.do_raise_r(tests, att='_setSelectorText')

    def test_reprANDstr(self):
        "SelectorList.__repr__(), .__str__()"
        sel=(u'a, p|b', { 'p': 'uri'})
        
        s = cssutils.css.SelectorList(selectorText=sel)

        self.assertTrue(sel[0] in str(s))

        s2 = eval(repr(s))
        self.assertTrue(isinstance(s2, s.__class__))
        self.assertEqual(sel[0], s2.selectorText)


if __name__ == '__main__':
    import unittest
    unittest.main()



"""Implements a DOM for MediaQuery, see 
http://www.w3.org/TR/css3-mediaqueries/.

A cssutils implementation, not defined in official DOM.
"""
__all__ = ['MediaQuery']
__docformat__ = 'restructuredtext'
__version__ = '$Id$'

from cssutils.prodparser import *
from cssutils.helper import normalize, pushtoken
import cssutils
import re
import xml.dom

class MediaQuery(cssutils.util._NewBase):#cssutils.util.Base):
    """
    A Media Query consists of one of :const:`MediaQuery.MEDIA_TYPES`
    and one or more expressions involving media features.

    Format::
  
        media_query
         : [ONLY | NOT]? S* media_type S* [ AND S* expression ]*
         | expression [ AND S* expression ]*
         ;
        media_type
         : IDENT
         ;
        expression
         : '(' S* media_feature S* [ ':' S* expr ]? ')' S*
         ;
        media_feature
         : IDENT
         ;
          
    """
    MEDIA_TYPES = ['all', 'braille', 'handheld', 'print', 'projection', 
                'speech', 'screen', 'tty', 'tv', 'embossed']


    def __init__(self, mediaText=None, readonly=False, _partof=False):
        """
        :param mediaText:
            unicodestring of parsable media

        # _standalone: True if new from ML parser
        """
        super(MediaQuery, self).__init__()

        self._wellformed = False
        self._mediaType = u''
        self._partof = _partof
        if mediaText:
            self.mediaText = mediaText # sets self._mediaType too
            self._partof = False

        self._readonly = readonly

    def __repr__(self):
        return "cssutils.stylesheets.%s(mediaText=%r)" % (
                self.__class__.__name__, self.mediaText)

    def __str__(self):
        return "<cssutils.stylesheets.%s object mediaText=%r at 0x%x>" % (
                self.__class__.__name__, self.mediaText, id(self))

    def _getMediaText(self):
        return cssutils.ser.do_stylesheets_mediaquery(self)

    def _setMediaText(self, mediaText):
        """
        :param mediaText:
            a single media query string, e.g. ``print and (min-width: 25cm)``

        :exceptions:    
            - :exc:`~xml.dom.SyntaxErr`:
              Raised if the specified string value has a syntax error and is
              unparsable.
            - :exc:`~xml.dom.InvalidCharacterErr`:
              Raised if the given mediaType is unknown.
            - :exc:`~xml.dom.NoModificationAllowedErr`:
              Raised if this media query is readonly.
        
        media_query
         : [ONLY | NOT]? S* media_type S* [ AND S* expression ]*
         | expression [ AND S* expression ]*
         ;
        media_type
         : IDENT
         ;
        expression
         : '(' S* media_feature S* [ ':' S* expr ]? ')' S*
         ;
        media_feature
         : IDENT
         ;

        """
        self._checkReadonly()

        expression = lambda: Sequence(PreDef.char(name='expression', char=u'('),
                                      Prod(name=u'media_feature',
                                           match=lambda t, v: t == PreDef.types.IDENT
                                      ),
                                      Sequence(PreDef.char(name='colon', char=u':'),
                                               cssutils.css.value.MediaQueryValueProd(self),
                                               minmax=lambda: (0, 1) # optional
                                               ),
                                      PreDef.char(name='expression END', char=u')',
                                                  stopIfNoMoreMatch=self._partof
                                                  )
                                      )

        prods = Choice(Sequence(Prod(name=u'ONLY|NOT', # media_query
                                     match=lambda t, v: t == PreDef.types.IDENT and 
                                                        normalize(v) in (u'only', u'not'),
                                     optional=True,
                                     toStore='not simple'
                                     ), 
                                Prod(name=u'media_type',
                                     match=lambda t, v: t == PreDef.types.IDENT and 
                                                        normalize(v) in self.MEDIA_TYPES,
                                     stopIfNoMoreMatch=True,
                                     toStore='media_type'
                                     ),                   
                                Sequence(Prod(name=u'AND',
                                              match=lambda t, v: t == PreDef.types.IDENT and 
                                                                 normalize(v) == 'and',
                                              toStore='not simple'
                                         ),                   
                                         expression(),
                                         minmax=lambda: (0, None)
                                         )
                                ),
                       Sequence(expression(),                   
                                Sequence(Prod(name=u'AND',
                                              match=lambda t, v: t == PreDef.types.IDENT and 
                                                                 normalize(v) == 'and'
                                         ),                   
                                         expression(),
                                         minmax=lambda: (0, None)
                                         )
                                )                        
                       )
        
        # parse
        ok, seq, store, unused = ProdParser().parse(mediaText, 
                                                    u'MediaQuery',
                                                    prods)
        self._wellformed = ok
        if ok:
            try:
                media_type = store['media_type']
            except KeyError, e:
                pass
            else:
                if 'not simple' not in store:
                    self.mediaType = media_type.value

            # TODO: filter doubles!
            self._setSeq(seq)

    mediaText = property(_getMediaText, _setMediaText,
        doc="The parsable textual representation of the media list.")

    def _setMediaType(self, mediaType):
        """
        :param mediaType:
            one of :attr:`MEDIA_TYPES`

        :exceptions:
            - :exc:`~xml.dom.SyntaxErr`:
              Raised if the specified string value has a syntax error and is
              unparsable.
            - :exc:`~xml.dom.InvalidCharacterErr`:
              Raised if the given mediaType is unknown.
            - :exc:`~xml.dom.NoModificationAllowedErr`:
              Raised if this media query is readonly.
        """
        self._checkReadonly()
        nmediaType = normalize(mediaType)

        if nmediaType not in self.MEDIA_TYPES:
            self._log.error(
                u'MediaQuery: Syntax Error in media type "%s".' % mediaType,
                error=xml.dom.SyntaxErr)
        else:
            # set
            self._mediaType = mediaType

            # update seq
            for i, x in enumerate(self._seq):
                if isinstance(x.value, basestring):
                    if normalize(x.value) in (u'only', u'not'):
                        continue
                    else:
                        # TODO: simplify!
                        self._seq[i] = (mediaType, 'IDENT', None, None)
                        break
            else:
                self._seq.insert(0, mediaType, 'IDENT')

    mediaType = property(lambda self: self._mediaType, _setMediaType,
        doc="The media type of this MediaQuery (one of "
            ":attr:`MEDIA_TYPES`) but only if it is a simple MediaType!")
    
    wellformed = property(lambda self: self._wellformed)



"""Testcases for cssutils.css.CSSStyleRuleTestCase"""

import xml.dom
import test_cssrule
import cssutils

class CSSStyleRuleTestCase(test_cssrule.CSSRuleTestCase):

    def setUp(self):
        super(CSSStyleRuleTestCase, self).setUp()
        self.r = cssutils.css.CSSStyleRule()
        self.rRO = cssutils.css.CSSStyleRule(readonly=True)
        self.r_type = cssutils.css.CSSStyleRule.STYLE_RULE
        self.r_typeString = 'STYLE_RULE'

    def test_init(self):
        "CSSStyleRule.type and init"
        super(CSSStyleRuleTestCase, self).test_init()
        self.assertEqual(u'', self.r.cssText)
        self.assertEqual(cssutils.css.selectorlist.SelectorList,
                         type(self.r.selectorList))
        self.assertEqual(u'', self.r.selectorText)
        self.assertEqual(cssutils.css.CSSStyleDeclaration,
                         type(self.r.style))
        self.assertEqual(self.r, self.r.style.parentRule)

    def test_refs(self):
        "CSSStyleRule references"
        s = cssutils.css.CSSStyleRule()
        sel, style = s.selectorList, s.style

        self.assertEqual(s, sel.parentRule)
        self.assertEqual(s, style.parentRule)

        s.cssText = 'a { x:1 }'
        self.assertNotEqual(sel, s.selectorList)
        self.assertEqual('a', s.selectorList.selectorText)
        self.assertNotEqual(style, s.style)
        self.assertEqual('1', s.style.getPropertyValue('x'))

        sel, style = s.selectorList, s.style

        invalids = ('$b { x:2 }', # invalid selector
                    'c { $x3 }', # invalid style
                    '/b { 2 }' # both invalid
                    )
        for invalid in invalids:
            try:
                s.cssText = invalid
            except xml.dom.DOMException, e:
                pass
            self.assertEqual(sel, s.selectorList)
            self.assertEqual(u'a', s.selectorList.selectorText)
            self.assertEqual(style, s.style)
            self.assertEqual(u'1', s.style.getPropertyValue('x'))

        # CHANGING
        s = cssutils.parseString(u'a {s1: 1}')
        r = s.cssRules[0]
        sel1 = r.selectorList
        st1 = r.style

        # selectorList
        r.selectorText = 'b'
        self.assertNotEqual(sel1, r.selectorList)
        self.assertEqual('b', r.selectorList.selectorText)
        self.assertEqual('b', r.selectorText)
        sel1b = r.selectorList

        sel1b.selectorText = 'c'
        self.assertEqual(sel1b, r.selectorList)
        self.assertEqual('c', r.selectorList.selectorText)
        self.assertEqual('c', r.selectorText)

        sel2 = cssutils.css.SelectorList('sel2')
        s.selectorList = sel2
        self.assertEqual(sel2, s.selectorList)
        self.assertEqual('sel2', s.selectorList.selectorText)

        sel2.selectorText = 'sel2b'
        self.assertEqual('sel2b', sel2.selectorText)
        self.assertEqual('sel2b', s.selectorList.selectorText)

        s.selectorList.selectorText = 'sel2c'
        self.assertEqual('sel2c', sel2.selectorText)
        self.assertEqual('sel2c', s.selectorList.selectorText)

        # style
        r.style = 's1: 2'
        self.assertNotEqual(st1, r.style)
        self.assertEqual('s1: 2', r.style.cssText)

        st2 = cssutils.parseStyle(u's2: 1')
        r.style = st2
        self.assertEqual(st2, r.style)
        self.assertEqual('s2: 1', r.style.cssText)

        # cssText
        sl, st = r.selectorList, r.style
        # fails
        try:
            r.cssText = '$ {content: "new"}'
        except xml.dom.SyntaxErr, e:
            pass
        self.assertEqual(sl, r.selectorList)
        self.assertEqual(st, r.style)

        r.cssText = 'a {content: "new"}'
        self.assertNotEqual(sl, r.selectorList)
        self.assertNotEqual(st, r.style)

    def test_cssText(self):
        "CSSStyleRule.cssText"
        tests = {
            u'* {}': u'',
            u'a {}': u'',
            }
        self.do_equal_p(tests) # parse
        #self.do_equal_r(tests) # set cssText # TODO: WHY?

        cssutils.ser.prefs.keepEmptyRules = True
        tests = {
            #u'''a{;display:block;float:left}''': 'a {\n    display:block;\n    float:left\n    }', # issue 28

            u'''a\n{color: #000}''': 'a {\n    color: #000\n    }', # issue 4
            u'''a\n{color: #000000}''': 'a {\n    color: #000\n    }', # issue 4
            u'''a\n{color: #abc}''': 'a {\n    color: #abc\n    }', # issue 4
            u'''a\n{color: #abcdef}''': 'a {\n    color: #abcdef\n    }', # issue 4
            u'''a\n{color: #00a}''': 'a {\n    color: #00a\n    }', # issue 4
            u'''a\n{color: #1a1a1a}''': 'a {\n    color: #1a1a1a\n    }', # issue 4
            u'''#id\n{ color: red }''': '#id {\n    color: red\n    }', # issue 3
            u'''* {}''': None,
            u'a {}': None,
            u'b { a: 1; }': u'b {\n    a: 1\n    }',
            # mix of comments and properties
            u'c1 {/*1*/a:1;}': u'c1 {\n    /*1*/\n    a: 1\n    }',
            u'c2 {a:1;/*2*/}': u'c2 {\n    a: 1;\n    /*2*/\n    }',
            u'd1 {/*0*/}': u'd1 {\n    /*0*/\n    }',
            u'd2 {/*0*//*1*/}': u'd2 {\n    /*0*/\n    /*1*/\n    }',
            # comments
            # TODO: spaces?
            u'''a/*1*//*2*/,/*3*//*4*/b/*5*//*6*/{color: #000}''':
                u'a/*1*//*2*/, /*3*//*4*/b/*5*//*6*/ {\n    color: #000\n    }',

            u'''a,b{color: #000}''': 'a, b {\n    color: #000\n    }', # issue 4
            u'''a\n\r\t\f ,\n\r\t\f b\n\r\t\f {color: #000}''': 'a, b {\n    color: #000\n    }', # issue 4
            }
        self.do_equal_p(tests) # parse
        self.do_equal_r(tests) # set cssText

        tests = {
            u'''a;''': xml.dom.SyntaxErr,
            u'''a {{}''': xml.dom.SyntaxErr,
            u'''a }''': xml.dom.SyntaxErr,
            }
        self.do_raise_p(tests) # parse
        tests.update({
            u'''/*x*/''': xml.dom.SyntaxErr,
            u'''a {''': xml.dom.SyntaxErr,
            # trailing
            u'''a {}x''': xml.dom.SyntaxErr,
            u'''a {/**/''': xml.dom.SyntaxErr,
            u'''a {} ''': xml.dom.SyntaxErr,
            })
        self.do_raise_r(tests) # set cssText
        cssutils.ser.prefs.useDefaults()

    def test_selectorList(self):
        "CSSStyleRule.selectorList"
        r = cssutils.css.CSSStyleRule()

        r.selectorList.appendSelector(u'a')
        self.assertEqual(1, r.selectorList.length)
        self.assertEqual(u'a', r.selectorText)

        r.selectorList.appendSelector(u' b  ')
        # only simple selector!
        self.assertRaises(xml.dom.InvalidModificationErr,
                          r.selectorList.appendSelector, u'  h1, x ')

        self.assertEqual(2, r.selectorList.length)
        self.assertEqual(u'a, b', r.selectorText)

    def test_selectorText(self):
        "CSSStyleRule.selectorText"
        r = cssutils.css.CSSStyleRule()

        r.selectorText = u'a'
        self.assertEqual(1, r.selectorList.length)
        self.assertEqual(u'a', r.selectorText)

        r.selectorText = u' b, h1  '
        self.assertEqual(2, r.selectorList.length)
        self.assertEqual(u'b, h1', r.selectorText)

    def test_style(self):
        "CSSStyleRule.style"
        d = cssutils.css.CSSStyleDeclaration()
        self.r.style = d
        self.assertEqual(d.cssText, self.r.style.cssText)

        # check if parentRule of d is set
        self.assertEqual(self.r, d.parentRule)

    def test_incomplete(self):
        "CSSStyleRule (incomplete)"
        cssutils.ser.prefs.keepEmptyRules = True
        tests = {
            u'a {': u'a {}', # no }
            u'a { font-family: "arial sans': # no "}
                u'a {\n    font-family: "arial sans"\n    }',
            u'a { font-family: "arial sans";': # no }
                u'a {\n    font-family: "arial sans"\n    }',
            u'''p {
                color: green;
                font-family: 'Courier New Times
                color: red;
                color: green;
                }''': u'''p {\n    color: green;\n    color: green\n    }''',
            # no ;
            u'''p {
                color: green;
                font-family: 'Courier New Times'
                color: red;
                color: green;
                ''': u'''p {\n    color: green;\n    color: green\n    }'''
        }
        self.do_equal_p(tests, raising=False) # parse
        cssutils.ser.prefs.useDefaults()

# TODO:   def test_InvalidModificationErr(self):
#        "CSSStyleRule.cssText InvalidModificationErr"
#        self._test_InvalidModificationErr(u'@a a {}')

    def test_reprANDstr(self):
        "CSSStyleRule.__repr__(), .__str__()"
        sel=u'a > b + c'

        s = cssutils.css.CSSStyleRule(selectorText=sel)

        self.assertTrue(sel in str(s))

        s2 = eval(repr(s))
        self.assertTrue(isinstance(s2, s.__class__))
        self.assertTrue(sel == s2.selectorText)

    def test_valid(self):
        "CSSStyleRule.valid"
        rule = cssutils.css.CSSStyleRule(selectorText='*', style='color: red')
        self.assertTrue(rule.valid)
        rule.style = 'color: foobar'
        self.assertFalse(rule.valid)
        rule.style = 'foobar: red'
        self.assertFalse(rule.valid)


if __name__ == '__main__':
    import unittest
    unittest.main()



import cssutils
import logging
cssutils.log.setLevel(logging.FATAL)


css = '''@import "example.css"; 
a {
    color: blue !important;
    c\olor: green !important;
    c\olor: pink;
    color: red;
    }'''

sheet = cssutils.parseString(css)
print "\nORIGINAL CSS:"
print css
print "------------"

print repr(cssutils.ser.prefs)

print "\nCSS Serialized"
print sheet.cssText

print "\nCSS Serialized with ``keepAllProperties`` = False" 
cssutils.ser.prefs.keepAllProperties = False
print sheet.cssText

print "\nCSS Serialized with ``defaultPropertyName`` = True"
cssutils.ser.prefs.defaultPropertyName = True
print sheet.cssText

print "\nCSS Serialized with ``defaultPropertyName`` = False"
cssutils.ser.prefs.defaultPropertyName = False
print sheet.cssText


print '\nBUT: Reading value programmatic uses normalized property'
print '     name and results in actual value only:'
print '\t.getPropertyValue("color") ==',
print sheet.cssRules[1].style.getPropertyValue('color')
print '\t.getPropertyValue("c\\olor") ==',
print sheet.cssRules[1].style.getPropertyValue('c\olor')
print '\t.getPropertyValue("c\\o\\l\\o\\r") ==',
print sheet.cssRules[1].style.getPropertyValue('c\\o\\l\\o\\r')
print 

print '\nCSS Serialized with indent = 2*" ", importHrefFormat="string", lineNumbers=True'
cssutils.ser.prefs.indent = 2*' '
# used to set indentation string, default is 4*' '
cssutils.ser.prefs.importHrefFormat = 'string'
# or 'uri', defaults to the format used in parsed stylesheet
cssutils.ser.prefs.lineNumbers = True
print sheet.cssText

print '\nCSS Serialized with useMinified()'
cssutils.ser.prefs.useMinified()
print sheet.cssText

# OUTPUTS
#1: @import "example.css";
#2: body {
#3:   color: red
#4:   }



"""Testcases for cssutils.css.CSSValue and CSSPrimitiveValue."""

#
## from decimal import Decimal # maybe for later tests?
#import xml.dom
#import basetest
#import cssutils
#import types
#
#class CSSValueTestCase(basetest.BaseTestCase):
#
#    def setUp(self):
#        self.r = cssutils.css.CSSValue() # needed for tests
#
#    def test_init(self):
#        "CSSValue.__init__()"
#        v = cssutils.css.CSSValue()
#        self.assertTrue(u'' == v.cssText)
#        self.assertTrue(None is  v.cssValueType)
#        self.assertTrue(None == v.cssValueTypeString)
#
#    def test_escapes(self):
#        "CSSValue Escapes"
#        v = cssutils.css.CSSValue()
#        v.cssText = u'1px'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_PX == v.primitiveType)
#        self.assertTrue(u'1px' == v.cssText)
#
#        v.cssText = u'1PX'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_PX == v.primitiveType)
#        self.assertTrue(u'1px' == v.cssText)
#        
#        v.cssText = u'1p\\x'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_PX == v.primitiveType)
#        self.assertTrue(u'1px' == v.cssText)
#
#    def test_cssText(self):
#        "CSSValue.cssText"
#        v = cssutils.css.CSSValue()
#        v.cssText = u'1px'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_PX == v.primitiveType)
#        self.assertTrue(u'1px' == v.cssText)
#
#        v = cssutils.css.CSSValue()
#        v.cssText = u'1px'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_PX == v.primitiveType)
#        self.assertTrue(u'1px' == v.cssText)
#
#        v = cssutils.css.CSSValue()
#        v.cssText = u'a  ,b,  c  ,"d or d", "e, " '
#        self.assertEqual(v.CSS_PRIMITIVE_VALUE, v.cssValueType)
#        self.assertEqual(v.CSS_STRING, v.primitiveType)
#        self.assertEqual(u'a, b, c, "d or d", "e, "', v.cssText)
#
#        v.cssText = u'  1   px    '
#        self.assertTrue(v.CSS_VALUE_LIST == v.cssValueType)
#        self.assertTrue('1 px' == v.cssText)
#
#        v.cssText = u'  normal 1px a, b, "c" end   '
#        self.assertTrue(v.CSS_VALUE_LIST == v.cssValueType)
#        self.assertEqual('normal 1px a, b, "c" end', v.cssText)
#
#        for i, x in enumerate(v):
#            self.assertEqual(x.CSS_PRIMITIVE_VALUE, x.cssValueType)
#            if x == 0:
#                self.assertEqual(x.CSS_IDENT, x.primitiveType)
#                self.assertEqual(u'normal', x.cssText)
#            elif x == 1:
#                self.assertEqual(x.CSS_PX, x.primitiveType)
#                self.assertEqual(u'1px', x.cssText)
#            if x == 2:
#                self.assertEqual(x.CSS_STRING, x.primitiveType)
#                self.assertEqual(u'a, b, "c"', x.cssText)
#            if x == 3:
#                self.assertEqual(x.CSS_IDENT, x.primitiveType)
#                self.assertEqual(u'end', x.cssText)
#
#
#        v = cssutils.css.CSSValue()
#        v.cssText = u'  1   px    '
#        self.assertTrue(v.CSS_VALUE_LIST == v.cssValueType)
#        self.assertTrue(u'1 px' == v.cssText)
#
#        v.cssText = u'expression(document.body.clientWidth > 972 ? "1014px": "100%" )'
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue(v.CSS_UNKNOWN == v.primitiveType)
#        self.assertEqual(u'expression(document.body.clientWidth > 972?"1014px": "100%")',
#                         v.cssText)
#
#    def test_cssText2(self):
#        "CSSValue.cssText 2"
#        tests = {
#            # mix
#            u'a()1,-1,+1,1%,-1%,1px,-1px,"a",a,url(a),#aabb44':
#                u'a() 1, -1, 1, 1%, -1%, 1px, -1px, "a", a, url(a), #ab4',
#            
#            # S or COMMENT
#            u'red': u'red',
#            u'red ': u'red',
#            u' red ': u'red',
#            u'/**/red': u'/**/ red',
#            u'red/**/': u'red /**/',
#            u'/**/red/**/': u'/**/ red /**/',
#            u'/**/ red': u'/**/ red',
#            u'red /**/': u'red /**/',
#            u'/**/ red /**/': u'/**/ red /**/',
#            u'red-': u'red-',
#            
#            # num / dimension 
#            u'.0': u'0',
#            u'0': u'0',
#            u'0.0': u'0',
#            u'00': u'0',
#            u'0%': u'0%',
#            u'0px': u'0',
#            u'-.0': u'0',
#            u'-0': u'0',
#            u'-0.0': u'0',
#            u'-00': u'0',
#            u'-0%': u'0%',
#            u'-0px': u'0',
#            u'+.0': u'0',
#            u'+0': u'0',
#            u'+0.0': u'0',
#            u'+00': u'0',
#            u'+0%': u'0%',
#            u'+0px': u'0',
#            u'1': u'1',
#            u'1.0': u'1',
#            u'1px': u'1px',
#            u'1%': u'1%',
#            u'1px1': u'1px1',
#            u'+1': u'1',
#            u'-1': u'-1',
#            u'+1.0': u'1',
#            u'-1.0': u'-1',
#            
#            # string, escaped nl is removed during tokenizing
#            u'"x"': u'"x"',
#            u"'x'": u'"x"',
#            #ur''' "1\'2" ''': u'''"1'2"''', #???
#            #ur"'x\"'": ur'"x\""', #???
#            ur'''"x\
#y"''': u'''"xy"''', 
#
#            # hash and rgb/a
#            u'#112234': u'#112234',
#            u'#112233': u'#123',
#            u'rgb(1,2,3)': u'rgb(1, 2, 3)',
#            u'rgb(  1  ,  2  ,  3  )': u'rgb(1, 2, 3)',
#            u'rgb(-1,+2,0)': u'rgb(-1, 2, 0)',
#            u'rgba(1,2,3,4)': u'rgba(1, 2, 3, 4)',
#            u'rgba(  1  ,  2  ,  3  ,  4 )': u'rgba(1, 2, 3, 4)',
#            u'rgba(-1,+2,0, 0)': u'rgba(-1, 2, 0, 0)',
#            
#            # FUNCTION 
#            u'f(1,2)': u'f(1, 2)',
#            u'f(  1  ,  2  )': u'f(1, 2)',
#            u'f(-1,+2)': u'f(-1, 2)',
#            u'f(  -1  ,  +2  )': u'f(-1, 2)',
#            u'fun(  -1  ,  +2  )': u'fun(-1, 2)',
#            u'local( x )': u'local(x)',
#            u'test(1px, #111, y, 1, 1%, "1", y(), var(x))': 
#                u'test(1px, #111, y, 1, 1%, "1", y(), var(x))',
#            u'test(-1px, #111, y, -1, -1%, "1", -y())': 
#                u'test(-1px, #111, y, -1, -1%, "1", -y())',
#            u'url(y)  format( "x" ,  "y" )': u'url(y) format("x", "y")',
#            u'f(1 2,3 4)': u'f(1 2, 3 4)',
#
#            # IE expression
#            ur'Expression()': u'Expression()',
#            ur'expression(-1 < +2)': u'expression(-1< + 2)',
#            ur'expression(document.width == "1")': u'expression(document.width=="1")',
#            u'alpha(opacity=80)': u'alpha(opacity=80)',
#            u'alpha( opacity = 80 , x=2  )': u'alpha(opacity=80, x=2)',
#
#            # unicode-range
#            'u+f': 'u+f',
#            'U+ABCdef': 'u+abcdef',
#
#            # url
#            'url(a)': 'url(a)',
#            'uRl(a)': 'url(a)',
#            'u\\rl(a)': 'url(a)',
#            'url("a")': 'url(a)',
#            'url(  "a"  )': 'url(a)',
#            'url(a)': 'url(a)',
#            'url(";")': 'url(";")',
#            'url(",")': 'url(",")',
#            'url(")")': 'url(")")',
#            '''url("'")''': '''url("'")''',
#            '''url('"')''': '''url("\\"")''',
#            '''url("'")''': '''url("'")''',
#            
#            # operator
#            '1': '1',
#            '1 2': '1 2',
#            '1   2': '1 2',
#            '1,2': '1, 2',
#            '1,  2': '1, 2',
#            '1  ,2': '1, 2',
#            '1  ,  2': '1, 2',
#            '1/2': '1/2',
#            '1/  2': '1/2',
#            '1  /2': '1/2',
#            '1  /  2': '1/2',
#             # comment
#            '1/**/2': '1 /**/ 2',
#            '1 /**/2': '1 /**/ 2',
#            '1/**/ 2': '1 /**/ 2',
#            '1 /**/ 2': '1 /**/ 2',
#            '1  /*a*/  /*b*/  2': '1 /*a*/ /*b*/ 2',
#            # , before
#            '1,/**/2': '1, /**/ 2',
#            '1 ,/**/2': '1, /**/ 2',
#            '1, /**/2': '1, /**/ 2',
#            '1 , /**/2': '1, /**/ 2',
#            # , after
#            '1/**/,2': '1 /**/, 2',
#            '1/**/ ,2': '1 /**/, 2',
#            '1/**/, 2': '1 /**/, 2',
#            '1/**/ , 2': '1 /**/, 2',
#            # all
#            '1/*a*/  ,/*b*/  2': '1 /*a*/, /*b*/ 2',
#            '1  /*a*/,  /*b*/2': '1 /*a*/, /*b*/ 2',
#            '1  /*a*/  ,  /*b*/  2': '1 /*a*/, /*b*/ 2',
#            
#            # list
#            'a b1,b2 b2,b3,b4': 'a b1, b2 b2, b3, b4',
#            'a b1  ,   b2   b2  ,  b3  ,   b4': 'a b1, b2 b2, b3, b4',
#            'u+1  ,   u+2-5': 'u+1, u+2-5',
#            u'local( x ),  url(y)  format( "x" ,  "y" )': 
#                u'local(x), url(y) format("x", "y")',
#            # FUNCTION
#            u'attr( href )': u'attr(href)',
#            # PrinceXML extende FUNC syntax with nested FUNC
#            u'target-counter(attr(href),page)': u'target-counter(attr(href), page)'
#            }
#
#        self.do_equal_r(tests)
#
#        tests = {
#            u'a+': xml.dom.SyntaxErr,
#            u'-': xml.dom.SyntaxErr,
#            u'+': xml.dom.SyntaxErr,
#            u'-%': xml.dom.SyntaxErr,
#            u'+a': xml.dom.SyntaxErr,
#            u'--1px': xml.dom.SyntaxErr,
#            u'++1px': xml.dom.SyntaxErr,
#            u'#': xml.dom.SyntaxErr,
#            u'#00': xml.dom.SyntaxErr,
#            u'#0000': xml.dom.SyntaxErr,
#            u'#00000': xml.dom.SyntaxErr,
#            u'#0000000': xml.dom.SyntaxErr,
#            u'-#0': xml.dom.SyntaxErr,
#            # operator
#            u',': xml.dom.SyntaxErr,
#            u'1,,2': xml.dom.SyntaxErr,
#            u'1,/**/,2': xml.dom.SyntaxErr,
#            u'1  ,  /**/  ,  2': xml.dom.SyntaxErr,
#            u'1,': xml.dom.SyntaxErr,
#            u'1, ': xml.dom.SyntaxErr,
#            u'1 ,': xml.dom.SyntaxErr,
#            u'1 , ': xml.dom.SyntaxErr,
#            u'1  ,  ': xml.dom.SyntaxErr,
#            u'1//2': xml.dom.SyntaxErr,
#            # URL
#            u'url(x))': xml.dom.SyntaxErr,
#            # string
#            u'"': xml.dom.SyntaxErr,
#            u"'": xml.dom.SyntaxErr,
#            # function 
#            u'f(-)': xml.dom.SyntaxErr,
#            u'f(x))': xml.dom.SyntaxErr
#            }
#        self.do_raise_r(tests)
#
#    def test_incomplete(self):
#        "CSSValue (incomplete)"
#        tests = {
#            u'url("a': u'url(a)',
#            u'url(a': u'url(a)'
#        }
#        for v, exp in tests.items():
#            s = cssutils.parseString('a { background: %s' % v)
#            v = s.cssRules[0].style.background
#            self.assertEqual(v, exp)
#
#    def test_cssValueType(self):
#        "CSSValue.cssValueType .cssValueTypeString"
#        tests = [
#            ([u'inherit', u'INhe\\rit'], 'CSS_INHERIT', cssutils.css.CSSValue),
#            (['1', '1%', '1em', '1ex', '1px', '1cm', '1mm', '1in', '1pt', '1pc',
#              '1deg', '1rad', '1grad', '1ms', '1s', '1hz', '1khz', '1other',
#               '"string"', "'string'", 'url(x)', 'red',
#               'attr(a)', 'counter(x)', 'rect(1px, 2px, 3px, 4px)',
#               'rgb(0, 0, 0)', '#000', '#123456', 'rgba(0, 0, 0, 0)',
#               'hsl(0, 0, 0)', 'hsla(0, 0, 0, 0)',
#               ],
#             'CSS_PRIMITIVE_VALUE', cssutils.css.CSSPrimitiveValue),
#            ([u'1px 1px', 'red blue green x'], 'CSS_VALUE_LIST', cssutils.css.CSSValueList),
#            # what is a custom value?
#            #([], 'CSS_CUSTOM', cssutils.css.CSSValue) 
#            ]
#        for values, name, cls in tests:
#            for value in values:
#                v = cssutils.css.CSSValue(cssText=value)
#                if value == "'string'":
#                    # will be changed to " always
#                    value = '"string"'
#                self.assertEqual(value, v.cssText)
#                self.assertEqual(name, v.cssValueTypeString)
#                self.assertEqual(getattr(v, name), v.cssValueType)
#                self.assertEqual(cls, type(v))
#
#    def test_readonly(self):
#        "(CSSValue._readonly)"
#        v = cssutils.css.CSSValue(cssText='inherit')
#        self.assertTrue(False is v._readonly)
#
#        v = cssutils.css.CSSValue(cssText='inherit', readonly=True)
#        self.assertTrue(True is v._readonly)
#        self.assertTrue(u'inherit', v.cssText)
#        self.assertRaises(xml.dom.NoModificationAllowedErr, v._setCssText, u'x')
#        self.assertTrue(u'inherit', v.cssText)
#
#    def test_reprANDstr(self):
#        "CSSValue.__repr__(), .__str__()"
#        cssText='inherit'
#
#        s = cssutils.css.CSSValue(cssText=cssText)
#
#        self.assertTrue(cssText in str(s))
#
#        s2 = eval(repr(s))
#        self.assertTrue(isinstance(s2, s.__class__))
#        self.assertTrue(cssText == s2.cssText)
#
#
#class CSSPrimitiveValueTestCase(basetest.BaseTestCase):
#
#    def test_init(self):
#        "CSSPrimitiveValue.__init__()"
#        v = cssutils.css.CSSPrimitiveValue(u'1')
#        self.assertTrue(u'1' == v.cssText)
#
#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
#        self.assertTrue("CSS_PRIMITIVE_VALUE" == v.cssValueTypeString)
#
#        self.assertTrue(v.CSS_NUMBER == v.primitiveType)
#        self.assertTrue("CSS_NUMBER" == v.primitiveTypeString)
#
#        # DUMMY to be able to test empty constructor call
#        #self.assertRaises(xml.dom.SyntaxErr, v.__init__, None)
#
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getCounterValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getRGBColorValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getRectValue)
#        self.assertRaises(xml.dom.InvalidAccessErr, v.getStringValue)
#
#    def test_CSS_UNKNOWN(self):
#        "CSSPrimitiveValue.CSS_UNKNOWN"
#        v = cssutils.css.CSSPrimitiveValue(u'expression(false)')
#        self.assertTrue(v.CSS_UNKNOWN == v.primitiveType)
#        self.assertTrue('CSS_UNKNOWN' == v.primitiveTypeString)
#
#    def test_CSS_NUMBER_AND_OTHER_DIMENSIONS(self):
#        "CSSPrimitiveValue.CSS_NUMBER .. CSS_DIMENSION"
#        defs = [
#            ('', 'CSS_NUMBER'),
#            ('%', 'CSS_PERCENTAGE'),
#            ('em', 'CSS_EMS'),
#            ('ex', 'CSS_EXS'),
#            ('px', 'CSS_PX'),
#            ('cm', 'CSS_CM'),
#            ('mm', 'CSS_MM'),
#            ('in', 'CSS_IN'),
#            ('pt', 'CSS_PT'),
#            ('pc', 'CSS_PC'),
#            ('deg', 'CSS_DEG'),
#            ('rad', 'CSS_RAD'),
#            ('grad', 'CSS_GRAD'),
#            ('ms', 'CSS_MS'),
#            ('s', 'CSS_S'),
#            ('hz', 'CSS_HZ'),
#            ('khz', 'CSS_KHZ'),
#            ('other_dimension', 'CSS_DIMENSION')
#            ]
#        for dim, name in defs:
#            for n in (0, 1, 1.1, -1, -1.1, -0):
#                v = cssutils.css.CSSPrimitiveValue('%i%s' % (n, dim))
#                self.assertEqual(name, v.primitiveTypeString)
#                self.assertEqual(getattr(v, name), v.primitiveType)
#
#    def test_CSS_STRING_AND_OTHER(self):
#        "CSSPrimitiveValue.CSS_STRING .. CSS_RGBCOLOR"
#        defs = [
#                (('""', "''", '"some thing"', "' A\\ND '",
#                  # comma separated lists are STRINGS FOR NOW!
#                  'a, b',
#                  '"a", "b"',
#                  ), 'CSS_STRING'),
#                (('url(a)', 'url("a b")', "url(' ')"), 'CSS_URI'),
#                (('some', 'or_anth-er'), 'CSS_IDENT'),            
#                (('attr(a)', 'attr(b)'), 'CSS_ATTR'),
#                (('counter(1)', 'counter(2)'), 'CSS_COUNTER'),
#                (('rect(1,2,3,4)',), 'CSS_RECT'),
#                (('rgb(1,2,3)', 'rgb(10%, 20%, 30%)', '#123', '#123456'),
#                 'CSS_RGBCOLOR'),
#                (('rgba(1,2,3,4)','rgba(10%, 20%, 30%, 40%)', ),
#                 'CSS_RGBACOLOR'),
#                (('U+0', 'u+ffffff', 'u+000000-f',
#                  'u+0-f, U+ee-ff'), 'CSS_UNICODE_RANGE')
#                ]
#
#        for examples, name in defs:
#            for x in examples:
#                v = cssutils.css.CSSPrimitiveValue(x)
#                self.assertEqual(getattr(v, name), v.primitiveType)
#                self.assertEqual(name, v.primitiveTypeString)
#
#    def test_getFloat(self):
#        "CSSPrimitiveValue.getFloatValue()"
#        # NOT TESTED are float values as it seems difficult to
#        # compare these. Maybe use decimal.Decimal?
#
#        v = cssutils.css.CSSPrimitiveValue(u'1px')
#        tests = {
#            '0': (v.CSS_NUMBER, 0),
#            '-1.1': (v.CSS_NUMBER, -1.1),
#            '1%': (v.CSS_PERCENTAGE, 1),
#            '-1%': (v.CSS_PERCENTAGE, -1),
#            '1em': (v.CSS_EMS, 1),
#            '-1.1em': (v.CSS_EMS, -1.1),
#            '1ex': (v.CSS_EXS, 1),
#            '1px': (v.CSS_PX, 1),
#
#            '1cm': (v.CSS_CM, 1),
#            '1cm': (v.CSS_MM, 10),
#            '254cm': (v.CSS_IN, 100),
#            '1mm': (v.CSS_MM, 1),
#            '10mm': (v.CSS_CM, 1),
#            '254mm': (v.CSS_IN, 10),
#            '1in': (v.CSS_IN, 1),
#            '100in': (v.CSS_CM, 254), # ROUNDED!!!
#            '10in': (v.CSS_MM, 254), # ROUNDED!!!
#
#            '1pt': (v.CSS_PT, 1),
#            '1pc': (v.CSS_PC, 1),
#
#            '1deg': (v.CSS_DEG, 1),
#            '1rad': (v.CSS_RAD, 1),
#            '1grad': (v.CSS_GRAD, 1),
#
#            '1ms': (v.CSS_MS, 1),
#            '1000ms': (v.CSS_S, 1),
#            '1s': (v.CSS_S, 1),
#            '1s': (v.CSS_MS, 1000),
#
#            '1hz': (v.CSS_HZ, 1),
#            '1000hz': (v.CSS_KHZ, 1),
#            '1khz': (v.CSS_KHZ, 1),
#            '1khz': (v.CSS_HZ, 1000),
#
#            '1DIMENSION': (v.CSS_DIMENSION, 1),            
#            }
#        for cssText in tests:
#            v.cssText = cssText
#            unitType, exp = tests[cssText]
#            val = v.getFloatValue(unitType)
#            if unitType in (v.CSS_IN, v.CSS_CM):
#                val = round(val)
#            self.assertEqual(val , exp)
#
#    def test_setFloat(self):
#        "CSSPrimitiveValue.setFloatValue()"
#        V = cssutils.css.CSSPrimitiveValue
#
#        tests = {
#            # unitType, value
#            (V.CSS_NUMBER, 1): [
#                # unitType, setvalue,
#                #    getvalue or expected exception, msg or cssText
#                (V.CSS_NUMBER, 0, 0, '0'),
#                (V.CSS_NUMBER, 0.1, 0.1, '0.1'),
#                (V.CSS_NUMBER, -0, 0, '0'),
#                (V.CSS_NUMBER, 2, 2, '2'),
#                (V.CSS_NUMBER, 2.0, 2, '2'),
#                (V.CSS_NUMBER, 2.1, 2.1, '2.1'),
#                (V.CSS_NUMBER, -2.1, -2.1, '-2.1'),
#                # setting with string does work
#                (V.CSS_NUMBER, '1', 1, '1'),
#                (V.CSS_NUMBER, '1.1', 1.1, '1.1'),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_RAD, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_GRAD, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_S, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_MS, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_KHZ, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_HZ, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DIMENSION, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_MM, 2, xml.dom.InvalidAccessErr, None),
#
#                (V.CSS_NUMBER, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: floatValue 'x' is not a float"),
#                (V.CSS_NUMBER, '1x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: floatValue '1x' is not a float"),
#
#                (V.CSS_STRING, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_STRING' is not a float type"),
#                (V.CSS_URI, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_URI' is not a float type"),
#                (V.CSS_ATTR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_ATTR' is not a float type"),
#                (V.CSS_IDENT, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_IDENT' is not a float type"),
#                (V.CSS_RGBCOLOR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RGBCOLOR' is not a float type"),
#                (V.CSS_RGBACOLOR, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RGBACOLOR' is not a float type"),
#                (V.CSS_RECT, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_RECT' is not a float type"),
#                (V.CSS_COUNTER, 'x', xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: unitType 'CSS_COUNTER' is not a float type"),
#                (V.CSS_EMS, 1, xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_NUMBER' to 'CSS_EMS'"),
#                (V.CSS_EXS, 1, xml.dom.InvalidAccessErr,
#                 "CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_NUMBER' to 'CSS_EXS'")
#            ],
#            (V.CSS_MM, '1mm'): [
#                (V.CSS_MM, 2, 2, '2mm'),
#                (V.CSS_MM, 0, 0, '0mm'),
#                (V.CSS_MM, 0.1, 0.1, '0.1mm'),
#                (V.CSS_MM, -0, -0, '0mm'),
#                (V.CSS_MM, 3.0, 3, '3mm'),
#                (V.CSS_MM, 3.1, 3.1, '3.1mm'),
#                (V.CSS_MM, -3.1, -3.1, '-3.1mm'),
#                (V.CSS_CM, 1, 10, '10mm'),
#                (V.CSS_IN, 10, 254, '254mm'),
#                (V.CSS_PT, 1, 1828.8, '1828.8mm'),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_NUMBER, 2, xml.dom.InvalidAccessErr, None)
#            ],
#            (V.CSS_PT, '1pt'): [
#                (V.CSS_PT, 2, 2, '2pt'),
#                (V.CSS_PC, 12, 1, '1pt'),
#                (V.CSS_NUMBER, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None)
#            ],
#            (V.CSS_KHZ, '1khz'): [
#                (V.CSS_HZ, 2000, 2, '2khz'),
#                (V.CSS_NUMBER, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_DEG, 1, xml.dom.InvalidAccessErr, None),
#                (V.CSS_PX, 1, xml.dom.InvalidAccessErr, None)
#            ]
#        }
#        for test in tests:
#            initialType, initialValue = test
#            pv = cssutils.css.CSSPrimitiveValue(initialValue)
#            for setType, setValue, exp, cssText in tests[test]:
#                if type(exp) == types.TypeType or\
#                   type(exp) == types.ClassType: # 2.4 compatibility
#                    if cssText:
#                        self.assertRaisesMsg(
#                            exp, cssText, pv.setFloatValue, setType, setValue)
#                    else:
#                        self.assertRaises(
#                            exp, pv.setFloatValue, setType, setValue)
#                else:
#                    pv.setFloatValue(setType, setValue)
#                    self.assertEqual(pv._value[0], cssText)
#                    if cssText == '0mm':
#                        cssText = '0'
#                    self.assertEqual(pv.cssText, cssText)
#                    self.assertEqual(pv.getFloatValue(initialType), exp)
#
#    def test_getString(self):
#        "CSSPrimitiveValue.getStringValue()"
#        v = cssutils.css.CSSPrimitiveValue(u'1px')
#        self.assertTrue(v.primitiveType == v.CSS_PX)
#        self.assertRaises(xml.dom.InvalidAccessErr,
#                          v.getStringValue)
#
#        pv = cssutils.css.CSSPrimitiveValue
#        tests = {
#            pv.CSS_STRING: ("'red'", 'red'),
#            pv.CSS_STRING: ('"red"', 'red'),
#            pv.CSS_URI: ('url(http://example.com)', None),
#            pv.CSS_URI: ("url('http://example.com')",
#                         u"http://example.com"),
#            pv.CSS_URI: ('url("http://example.com")',
#                         u'http://example.com'),
#            pv.CSS_URI: ('url("http://example.com?)")',
#                         u'http://example.com?)'),
#            pv.CSS_IDENT: ('red', None),
#            pv.CSS_ATTR: ('attr(att-name)',
#                         u'att-name'), # the name of the attrr
#            }
#        for t in tests:
#            val, exp = tests[t]
#            if not exp:
#                exp = val
#
#            v = cssutils.css.CSSPrimitiveValue(val)
#            self.assertEqual(v.primitiveType, t)
#            self.assertEqual(v.getStringValue(), exp)
#
#    def test_setString(self):
#        "CSSPrimitiveValue.setStringValue()"
#        # CSS_STRING
#        v = cssutils.css.CSSPrimitiveValue(u'"a"')
#        self.assertTrue(v.CSS_STRING == v.primitiveType)
#        v.setStringValue(v.CSS_STRING, 'b')
#        self.assertTrue(('b', 'STRING') == v._value)
#        self.assertEqual('b', v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_IDENT
#        v = cssutils.css.CSSPrimitiveValue('new')
#        v.setStringValue(v.CSS_IDENT, 'ident')
#        self.assertTrue(v.CSS_IDENT == v.primitiveType)
#        self.assertTrue(('ident', 'IDENT') == v._value)
#        self.assertTrue('ident' == v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_URI
#        v = cssutils.css.CSSPrimitiveValue('url(old)')
#        v.setStringValue(v.CSS_URI, '(')
#        self.assertEqual((u'(', 'URI'), v._value)
#        self.assertEqual(u'(', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, ')')
#        self.assertEqual((u')', 'URI'), v._value)
#        self.assertEqual(u')', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, '"') 
#        self.assertEqual(ur'"', v.getStringValue())
#        self.assertEqual((ur'"', 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, "''") 
#        self.assertEqual(ur"''", v.getStringValue())
#        self.assertEqual((ur"''", 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, ',') 
#        self.assertEqual(ur',', v.getStringValue())
#        self.assertEqual((ur',', 'URI'), v._value)
#
#        v.setStringValue(v.CSS_URI, ' ')
#        self.assertEqual((u' ', 'URI'), v._value)
#        self.assertEqual(u' ', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, 'a)')
#        self.assertEqual((u'a)', 'URI'), v._value)
#        self.assertEqual(u'a)', v.getStringValue())
#
#        v.setStringValue(v.CSS_URI, 'a')
#        self.assertTrue(v.CSS_URI == v.primitiveType)
#        self.assertEqual((u'a', 'URI'), v._value)
#        self.assertEqual(u'a', v.getStringValue())
#
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_URI' to 'CSS_ATTR'",
#            v.setStringValue, *(v.CSS_ATTR, 'x'))
#
#        # CSS_ATTR
#        v = cssutils.css.CSSPrimitiveValue('attr(old)')
#        v.setStringValue(v.CSS_ATTR, 'a')
#        self.assertTrue(v.CSS_ATTR == v.primitiveType)
#        self.assertTrue('a' == v.getStringValue())
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_IDENT'",
#            v.setStringValue, *(v.CSS_IDENT, 'x'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_STRING'",
#            v.setStringValue, *(v.CSS_STRING, '"x"'))
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_URI'",
#            v.setStringValue, *(v.CSS_URI, 'x'))
#
#        # TypeError as 'x' is no valid type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType 'x' (UNKNOWN TYPE) is not a string type",
#            v.setStringValue, *('x', 'brown'))
#        # IndexError as 111 is no valid type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType 111 (UNKNOWN TYPE) is not a string type",
#            v.setStringValue, *(111, 'brown'))
#        # CSS_PX is no string type
#        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
#            u"CSSPrimitiveValue: stringType CSS_PX is not a string type",
#            v.setStringValue, *(v.CSS_PX, 'brown'))
#
#    def test_typeRGBColor(self):
#        "RGBColor"
#        v = cssutils.css.CSSPrimitiveValue('RGB(1, 5, 10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1, 5, 10)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('rgb(1, 5, 10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1, 5, 10)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('rgb(1%, 5%, 10%)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        self.assertEqual(u'rgb(1%, 5%, 10%)', v.cssText)
#
#        v = cssutils.css.CSSPrimitiveValue('  rgb(  1 ,5,  10  )')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        v = cssutils.css.CSSPrimitiveValue('rgb(1,5,10)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#        v = cssutils.css.CSSPrimitiveValue('rgb(1%, .5%, 10.1%)')
#        self.assertEqual(v.CSS_RGBCOLOR, v.primitiveType)
#
#    def test_reprANDstr(self):
#        "CSSPrimitiveValue.__repr__(), .__str__()"
#        v='111'
#
#        s = cssutils.css.CSSPrimitiveValue(v)
#
#        self.assertTrue(v in str(s))
#        self.assertTrue('CSS_NUMBER' in str(s))
#
#        s2 = eval(repr(s))
#        self.assertTrue(isinstance(s2, s.__class__))
#        self.assertTrue(v == s2.cssText)
#
#
#class CSSValueListTestCase(basetest.BaseTestCase):
#
#    def test_init(self):
#        "CSSValueList.__init__()"
#        v = cssutils.css.CSSValue(cssText=u'red blue')
#        self.assertTrue(v.CSS_VALUE_LIST == v.cssValueType)
#        self.assertEqual('red blue', v.cssText)
#
#        self.assertTrue(2 == v.length)
#
#        item = v.item(0)
#        item.setStringValue(item.CSS_IDENT, 'green')
#        self.assertEqual('green blue', v.cssText)
#
#    def test_numbers(self):
#        "CSSValueList.cssText"
#        tests = {
#            u'0 0px -0px +0px': (u'0 0 0 0', 4),
#            u'1 2 3 4': (None, 4),
#            u'-1 -2 -3 -4': (None, 4),
#            u'-1 2': (None, 2),
#            u'-1px red "x"': (None, 3),
#            u'a, b c': (None, 2),
#            u'1px1 2% 3': (u'1px1 2% 3', 3),
#            u'f(+1pX, -2, 5%) 1': (u'f(1px, -2, 5%) 1', 2),
#            u'0 f()0': (u'0 f() 0', 3),
#            u'f()0': (u'f() 0', 2),
#            u'f()1%': (u'f() 1%', 2),
#            u'f()1px': (u'f() 1px', 2),
#            u'f()"str"': (u'f() "str"', 2),
#            u'f()ident': (u'f() ident', 2),
#            u'f()#123': (u'f() #123', 2),
#            u'f()url()': (u'f() url()', 2),
#            u'f()f()': (u'f() f()', 2),
#            u'url(x.gif)0 0': (u'url(x.gif) 0 0', 3),
#            u'url(x.gif)no-repeat': (u'url(x.gif) no-repeat', 2)
#            }
#        for test in tests:
#            exp, num = tests[test]
#            if not exp:
#                exp = test
#            v = cssutils.css.CSSValue(cssText=test)
#            self.assertTrue(v.CSS_VALUE_LIST == v.cssValueType)
#            self.assertEqual(num, v.length)
#            self.assertEqual(exp, v.cssText)
#
#    def test_reprANDstr(self):
#        "CSSValueList.__repr__(), .__str__()"
#        v='1px 2px'
#
#        s = cssutils.css.CSSValue(v)
#        self.assertTrue(isinstance(s, cssutils.css.CSSValueList))
#
#        self.assertTrue('length=2' in str(s))
#        self.assertTrue(v in str(s))
#
#        # not "eval()"able!
#        #s2 = eval(repr(s))
#
#
#if __name__ == '__main__':
#    import unittest
#    unittest.main()



import cssutils
from cssutils import css, stylesheets

EXPOUT = u'''--- ORIGINAL ---\n@charset "ascii";\n        @namespace PREfix "uri";\n        SOME > WeIrD + selector ~ used here {color: green}\n        PREfix|name {color: green}\n    \n\n--- SELECTORS TO LOWER CASE (does not simply work for PREfix|name!) ---\n--- CHANGE PREFIX (prefix is not really part of selectorText, URI is! ---\n\n@charset "ascii";\n@namespace lower-case_prefix "uri";\nsome > weird + selector ~ used here {\n    color: green\n    }\nlower-case_prefix|name {\n    color: green\n    }\n'''
EXPERR = u'Property: Found valid "CSS Level 2.1" value: green [3:46: color]\nProperty: Found valid "CSS Level 2.1" value: green [4:22: color]\n'

def main():
    examplecss = u"""@charset "ascii";
        @namespace PREfix "uri";
        SOME > WeIrD + selector ~ used here {color: green}
        PREfix|name {color: green}
    """
    
    import logging
    sheet = cssutils.CSSParser(loglevel=logging.DEBUG).parseString(examplecss)
    
    print "--- ORIGINAL ---"
    print examplecss
    print
    
    print "--- SELECTORS TO LOWER CASE (does not simply work for PREfix|name!) ---"
    sheet.cssRules[2].selectorText = sheet.cssRules[2].selectorText.lower() 
    
    print "--- CHANGE PREFIX (prefix is not really part of selectorText, URI is! ---"
    sheet.cssRules[1].prefix = 'lower-case_prefix'
    
    print
    print sheet.cssText

if __name__ == '__main__':
    main()


"""CSSMediaRule implements DOM Level 2 CSS CSSMediaRule."""
__all__ = ['CSSMediaRule']
__docformat__ = 'restructuredtext'
__version__ = '$Id$'

import cssrule
import cssutils
import xml.dom

class CSSMediaRule(cssrule.CSSRuleRules):
    """
    Objects implementing the CSSMediaRule interface can be identified by the
    MEDIA_RULE constant. On these objects the type attribute must return the
    value of that constant.

    Format::

      : MEDIA_SYM S* medium [ COMMA S* medium ]* 
      
          STRING? # the name
      
      LBRACE S* ruleset* '}' S*;
      
    ``cssRules``
        All Rules in this media rule, a :class:`~cssutils.css.CSSRuleList`.
    """
    def __init__(self, mediaText='all', name=None,
                 parentRule=None, parentStyleSheet=None, readonly=False):
        """constructor"""
        super(CSSMediaRule, self).__init__(parentRule=parentRule, 
                                           parentStyleSheet=parentStyleSheet)
        self._atkeyword = u'@media'
        
        # 1. media 
        if mediaText:
            self.media = mediaText
        else:
            self.media = cssutils.stylesheets.MediaList()

        self.name = name
        self._readonly = readonly

    def __repr__(self):
        return u"cssutils.css.%s(mediaText=%r)" % (
                self.__class__.__name__,
                self.media.mediaText)
        
    def __str__(self):
        return u"<cssutils.css.%s object mediaText=%r at 0x%x>" % (
                self.__class__.__name__,
                self.media.mediaText,
                id(self))

    def _getCssText(self):
        """Return serialized property cssText."""
        return cssutils.ser.do_CSSMediaRule(self)

    def _setCssText(self, cssText):
        """
        :param cssText:
            a parseable string or a tuple of (cssText, dict-of-namespaces)
        :Exceptions:
            - :exc:`~xml.dom.NamespaceErr`:
              Raised if a specified selector uses an unknown namespace
              prefix.
            - :exc:`~xml.dom.SyntaxErr`:
              Raised if the specified CSS string value has a syntax error and
              is unparsable.
            - :exc:`~xml.dom.InvalidModificationErr`:
              Raised if the specified CSS string value represents a different
              type of rule than the current one.
            - :exc:`~xml.dom.HierarchyRequestErr`:
              Raised if the rule cannot be inserted at this point in the
              style sheet.
            - :exc:`~xml.dom.NoModificationAllowedErr`:
              Raised if the rule is readonly.
        """
        # media "name"? { cssRules }
        super(CSSMediaRule, self)._setCssText(cssText)
        
        # might be (cssText, namespaces)
        cssText, namespaces = self._splitNamespacesOff(cssText)
        
        try:
            # use parent style sheet ones if available
            namespaces = self.parentStyleSheet.namespaces
        except AttributeError:
            pass
        
        tokenizer = self._tokenize2(cssText)
        attoken = self._nexttoken(tokenizer, None)
        if self._type(attoken) != self._prods.MEDIA_SYM:
            self._log.error(u'CSSMediaRule: No CSSMediaRule found: %s' %
                self._valuestr(cssText),
                error=xml.dom.InvalidModificationErr)
        
        else:            
            # save if parse goes wrong
            oldMedia = self._media
            oldName = self._name
            oldCssRules = self._cssRules
            
            ok = True

            # media
            mediatokens, end = self._tokensupto2(tokenizer, 
                                                 mediaqueryendonly=True,
                                                 separateEnd=True)        
            if u'{' == self._tokenvalue(end)\
               or self._prods.STRING == self._type(end):
                self.media = cssutils.stylesheets.MediaList(parentRule=self)
                # TODO: remove special case
                self.media.mediaText = mediatokens
                ok = ok and self.media.wellformed
            else:
                ok = False
            
            # name (optional)
            name = None
            nameseq = self._tempSeq()
            if self._prods.STRING == self._type(end):
                name = self._stringtokenvalue(end)
                # TODO: for now comments are lost after name
                nametokens, end = self._tokensupto2(tokenizer, 
                                                    blockstartonly=True,
                                                    separateEnd=True)
                wellformed, expected = self._parse(None, 
                                                   nameseq, 
                                                   nametokens, 
                                                   {})
                if not wellformed:
                    ok = False
                    self._log.error(u'CSSMediaRule: Syntax Error: %s' % 
                                    self._valuestr(cssText))
                    

            # check for {
            if u'{' != self._tokenvalue(end):
                self._log.error(u'CSSMediaRule: No "{" found: %s' % 
                                self._valuestr(cssText))
                return
            
            # cssRules
            cssrulestokens, braceOrEOF = self._tokensupto2(tokenizer, 
                                               mediaendonly=True,
                                               separateEnd=True)
            nonetoken = self._nexttoken(tokenizer, None)
            if 'EOF' == self._type(braceOrEOF):
                # HACK!!!
                # TODO: Not complete, add EOF to rule and } to @media
                cssrulestokens.append(braceOrEOF)
                braceOrEOF = ('CHAR', '}', 0, 0)
                self._log.debug(u'CSSMediaRule: Incomplete, adding "}".', 
                                token=braceOrEOF, neverraise=True)

            if u'}' != self._tokenvalue(braceOrEOF):
                self._log.error(u'CSSMediaRule: No "}" found.', 
                                token=braceOrEOF)
            elif nonetoken:
                self._log.error(u'CSSMediaRule: Trailing content found.',
                                token=nonetoken)
            else:                
                # for closures: must be a mutable
                new = {'wellformed': True }
                                   
                def COMMENT(expected, seq, token, tokenizer=None):
                    self.insertRule(cssutils.css.CSSComment([token],
                                                            parentRule=self,
                                                            parentStyleSheet=self.parentStyleSheet))
                    return expected

                def ruleset(expected, seq, token, tokenizer):
                    rule = cssutils.css.CSSStyleRule(parentRule=self,
                                                     parentStyleSheet=self.parentStyleSheet)
                    rule.cssText = self._tokensupto2(tokenizer, token)
                    if rule.wellformed:
                        self.insertRule(rule)
                    return expected
        
                def atrule(expected, seq, token, tokenizer):
                    # TODO: get complete rule!
                    tokens = self._tokensupto2(tokenizer, token)
                    atval = self._tokenvalue(token)
                    factories = {
                        '@page': cssutils.css.CSSPageRule,
                        '@media': CSSMediaRule,
                    }
                    if atval in ('@charset ', '@font-face', '@import', 
                                 '@namespace', '@variables'):
                        self._log.error(u'CSSMediaRule: This rule is not '
                                        u'allowed in CSSMediaRule - ignored: '
                                        u'%s.' % self._valuestr(tokens),
                                        token = token, 
                                        error=xml.dom.HierarchyRequestErr)
                    elif atval in factories:
                        rule = factories[atval](
                            parentRule=self,
                            parentStyleSheet=self.parentStyleSheet)
                        rule.cssText = tokens
                        if rule.wellformed:
                            self.insertRule(rule)
                    else:
                        rule = cssutils.css.CSSUnknownRule(tokens,
                                                           parentRule=self, 
                                        parentStyleSheet=self.parentStyleSheet)
                        if rule.wellformed:
                            self.insertRule(rule)
                    return expected
                
                # save for possible reset
                oldCssRules = self.cssRules
                
                self.cssRules = cssutils.css.CSSRuleList()
                seq = [] # not used really

                tokenizer = iter(cssrulestokens)
                wellformed, expected = self._parse(braceOrEOF,
                                                   seq,
                                                   tokenizer, {
                                                     'COMMENT': COMMENT,
                                                     'CHARSET_SYM': atrule,
                                                     'FONT_FACE_SYM': atrule,
                                                     'IMPORT_SYM': atrule,
                                                     'NAMESPACE_SYM': atrule,
                                                     'PAGE_SYM': atrule,
                                                     'MEDIA_SYM': atrule,
                                                     'ATKEYWORD': atrule
                                                   }, 
                                                   default=ruleset,
                                                   new=new)
                ok = ok and wellformed
                
            if ok:
                self.name = name
                self._setSeq(nameseq)
            else:
                self._media = oldMedia
                self._cssRules = oldCssRules
                                            
    cssText = property(_getCssText, _setCssText,
                       doc=u"(DOM) The parsable textual representation of this "
                           u"rule.")

    def _setName(self, name):
        if isinstance(name, basestring) or name is None:
            # "" or ''
            if not name:
                name = None

            self._name = name
        else:
            self._log.error(u'CSSImportRule: Not a valid name: %s' % name)

    name = property(lambda self: self._name, _setName,
                    doc=u"An optional name for this media rule.")

    def _setMedia(self, media):
        """
        :param media:
            a :class:`~cssutils.stylesheets.MediaList` or string
        """
        self._checkReadonly()
        if isinstance(media, basestring):
            self._media = cssutils.stylesheets.MediaList(mediaText=media, 
                                                         parentRule=self)
        else:
            media._parentRule = self
            self._media = media
        
        # NOT IN @media seq at all?!
#        # update seq
#        for i, item in enumerate(self.seq):
#            if item.type == 'media':
#                self._seq[i] = (self._media, 'media', None, None)
#                break
#        else:
#            # insert after @media if not in seq at all
#            self.seq.insert(0, 
#                             self._media, 'media', None, None)
            
    media = property(lambda self: self._media, _setMedia,
                     doc=u"(DOM) A list of media types for this rule "
                         u"of type :class:`~cssutils.stylesheets.MediaList`.")
    
            
    def insertRule(self, rule, index=None):
        """Implements base ``insertRule``."""
        rule, index = self._prepareInsertRule(rule, index)
        
        if rule is False or rule is True:
            # done or error
            return
        
        # check hierarchy
        if isinstance(rule, cssutils.css.CSSCharsetRule) or \
           isinstance(rule, cssutils.css.CSSFontFaceRule) or \
           isinstance(rule, cssutils.css.CSSImportRule) or \
           isinstance(rule, cssutils.css.CSSNamespaceRule) or \
           isinstance(rule, cssutils.css.MarginRule):
            self._log.error(u'%s: This type of rule is not allowed here: %s' 
                            % (self.__class__.__name__, rule.cssText),
                            error=xml.dom.HierarchyRequestErr)
            return

        return self._finishInsertRule(rule, index)

    type = property(lambda self: self.MEDIA_RULE, 
                    doc=u"The type of this rule, as defined by a CSSRule "
                        u"type constant.")

    wellformed = property(lambda self: self.media.wellformed)



# -*- coding: utf-8 -*-
import cssutils
import os.path
import xml.dom

EXPOUT = u'''@charset "iso-8859-1";
@import "1inherit-iso.css";

##############################
1inherit-iso.css
##############################
@charset "iso-8859-1";
@import "2inherit-iso.css";
/* 1 inherited encoding iso-8859-1 */

====================
2inherit-iso.css
====================
@charset "iso-8859-1";
/* 2 inherited encoding iso-8859-1 */



'''
EXPERR = u''


def main():

    def p(s, l=0):
        c = '#=-'[l] * (30 - l*10)
        for r in s.cssRules.rulesOfType(cssutils.css.CSSRule.IMPORT_RULE):
            print c
            print r.href
            print c
            print r.styleSheet.cssText
            print 
            p(r.styleSheet, l=l+1)
            print 
            
    s = cssutils.parseFile(os.path.join('sheets', '1import.css'))        
    print s.cssText
    print
    
    p(s)

if __name__ == '__main__':
    main()


import logging, StringIO

EXPOUT = "@variables{c:#0f0}a{color:var(c)}\na{color:#0f0}\n"
EXPERR = u'Property: Found valid "CSS Level 2.1" value: #0f0 [6:9: color]\n'

def main():
    import cssutils
    import cssutils.script
    
    css = '''
    @variables {
        c: #0f0;
    }
    a {
        color: var(c);
    }
    '''
    s = cssutils.parseString(css)
    
    cssutils.ser.prefs.useMinified()
    cssutils.ser.prefs.resolveVariables = False
    print s.cssText

    # reset
    cssutils.ser.prefs.resolveVariables = True
    print s.cssText

    
if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
"""test utils for tests of examples

A module to test must have:

- main(): a method
- EXPOUT: string which contains expected output
- EXPERR: string which contains expected error output
"""
__all__ = ['TestUtil']

import logging
import os
import StringIO
import sys

import cssutils

PY2x = sys.version_info < (3,0)

class OutReplacement(object):
    "io.StringIO does not work somehow?!"
    def __init__(self):
        self.t = ''
        
    def write(self, t):
        if not PY2x:
            # py3 hack
            if t.startswith('b') and t.endswith("'"):
                t = t[2:-1]
                
        self.t += t
        
    def getvalue(self):
        if not PY2x:
            # py3 hack
            self.t = self.t.replace('\\n', '\n')
            self.t = self.t.replace('\\\\', '\\')
            
        return self.t
    

class TestUtil(object):
    def __init__(self):
        "init out and err to catch both"
        self._out = OutReplacement()#StringIO()
        sys.stdout = self._out
        
        self._err = StringIO.StringIO()
        hdlr = logging.StreamHandler(self._err)
        cssutils.log._log.addHandler(hdlr)
                
    def end(self, expo, expe):
        "return out, err"
        sys.stdout = sys.__stdout__

        out = self._out.getvalue()
        err = self._err.getvalue()

        ok = (out == expo and err == expe)
        if not ok:
            print
            if out != expo:
                print '### out:\n%r\n### != expout:\n%r\n' % (out, expo)
            else:
                print '### err:\n%r\n### != experr:\n%r\n' % (err, expe)
        return ok

modules = 0
errors = 0

def mod(module):
    global modules, errors
    modules += 1
    t = TestUtil()
    module.main()
    ok = t.end(module.EXPOUT, module.EXPERR)
    if not ok:
        errors += 1
    print '---', ok, ':', os.path.basename(module.__file__)
    print 
          
def main():
    if PY2x:
        print "DOCTESTS:::::::::::::"
        # doctests
        import website
        import doctest
        doctest.testmod(website)
        print 
        print 80*'-' 
        print 
        print 

    global modules, errors
    
    import build
    mod(build)
    import customlog
    mod(customlog)
    import imports
    mod(imports) 
    import parse
    mod(parse)
    import selectors_tolower
    mod(selectors_tolower)

    import cssencodings
    mod(cssencodings)
    
    import minify
    mod(minify)
    
    print 
    print 80*'-' 
    print 'Ran %i tests (%i errors).' % (modules, errors)
    if PY2x:
        print '**Check doctest results above!**'
    else:
        print 'doctests do not work in Python 3 (yet?)!'
          
    
if __name__ == '__main__':
    main()
